# -*- coding: utf-8 -*-
import matplotlib as mpl
import numpy as np

import pymoskito as pm

from . import settings as st


class CarVisualizer(pm.MplVisualizer):
    def __init__(self, q_widget, q_layout):
        pm.MplVisualizer.__init__(self, q_widget, q_layout)
        self.image = None
        self.color = "green"
        self.axes.set_xlim(-1, 1)
        self.axes.set_ylim(-0, 2)
        self.axes.set_aspect('equal')
        self.axes.set_xlabel(r'$\xi_1/m$')
        self.axes.set_ylabel(r'$\xi_2/m$')
        self.update_scene(st.initial_state)

    def update_scene(self, x):
        x1, x2, theta1, theta2, theta3 = x
        d1, l2, d2, l3 = st.d1, st.l2, st.d2, st.l3
        dia = st.dia
        car_radius = st.dia / 2
        wheel = st.dia / 4
        ct1 = np.cos(theta1)
        st1 = np.sin(theta1)
        ct2 = np.cos(theta2)
        st2 = np.sin(theta2)
        ct3 = np.cos(theta3)
        st3 = np.sin(theta3)

        [xR1_1, yR1_1, xR1_2, yR1_2, xR2_1, yR2_1, xR2_2, yR2_2,
         x1_joint1, x2_joint1,
         x1_trailer1, x2_trailer1,
         x1T1_1, y1T1_1, x1T1_2, y1T1_2, x1T2_1, y1T2_1, x1T2_2, y1T2_2,
         x1_joint2, x2_joint2,
         x1_trailer2, x2_trailer2,
         x2T1_1, y2T1_1, x2T1_2, y2T1_2, x2T2_1, y2T2_1, x2T2_2, y2T2_2
         ] = self.calc_positions(
            x1, x2, theta1, theta2, theta3, dia, d1, l2, d2, l3, car_radius, wheel, ct1, st1, ct2, st2, ct3, st3)

        if self.image is None:
            # chassis
            sphere = mpl.patches.Circle((x1, x2), car_radius, color=self.color, zorder=0)
            self.axes.add_patch(sphere)

            # wheel 1
            wheel_1 = self.axes.add_line(
                mpl.lines.Line2D([xR1_1, xR1_2], [yR1_1, yR1_2], color='k', zorder=1, linewidth=3.0))

            # wheel 2
            wheel_2 = self.axes.add_line(
                mpl.lines.Line2D([xR2_1, xR2_2], [yR2_1, yR2_2], color='k', zorder=1, linewidth=3.0))
            track = None

            # rod 1
            rod_1 = self.axes.add_line(
                mpl.lines.Line2D([x1, x1_joint1], [x2, x2_joint1], color='k', zorder=3, linewidth=2.0))

            # rod 2
            rod_2 = self.axes.add_line(
                mpl.lines.Line2D([x1_joint1, x1_trailer1], [x2_joint1, x2_trailer1], color='k', zorder=3,
                                 linewidth=2.0))

            # rod 3
            rod_3 = self.axes.add_line(
                mpl.lines.Line2D([x1_trailer1, x1_joint2], [x2_trailer1, x2_joint2], color='k', zorder=3,
                                 linewidth=2.0))

            # rod 4
            rod_4 = self.axes.add_line(
                mpl.lines.Line2D([x1_joint2, x1_trailer2], [x2_joint2, x2_trailer2], color='k', zorder=3,
                                 linewidth=2.0))

            # joint 1
            joint_1 = mpl.patches.Circle((x1_joint1, x2_joint1), 0.01, color='k', zorder=3)
            self.axes.add_patch(joint_1)

            # joint 2
            joint_2 = mpl.patches.Circle((x1_joint2, x2_joint2), 0.01, color='k', zorder=3)
            self.axes.add_patch(joint_2)

            # trailer 1
            trailer1 = mpl.patches.FancyBboxPatch((x1_trailer1 - dia * 0.25, x2_trailer1 - dia * 0.25), 0.5 * dia,
                                                  0.5 * dia, color='0.5', zorder=0, fill=True, mutation_scale=0.05)
            t_start = self.axes.transData
            t = mpl.transforms.Affine2D().rotate_around(x1_trailer1, x2_trailer1, theta2)
            t_end = t + t_start
            trailer1.set_transform(t_end)
            self.axes.add_patch(trailer1)

            wheel_t11 = self.axes.add_line(
                mpl.lines.Line2D([x1T1_1, x1T1_2], [y1T1_1, y1T1_2], color='k', zorder=1, linewidth=3.0))

            wheel_t12 = self.axes.add_line(
                mpl.lines.Line2D([x1T2_1, x1T2_2], [y1T2_1, y1T2_2], color='k', zorder=1, linewidth=3.0))

            axis1 = self.axes.add_line(
                mpl.lines.Line2D([x1_trailer1 + 3. / 8. * dia * st2, x1_trailer1 + 0.5 * dia * st2],
                                 [x2_trailer1 - 3. / 8. * dia * ct2, x2_trailer1 - 0.5 * dia * ct2], color='k',
                                 zorder=1, linewidth=2.0))

            axis2 = self.axes.add_line(
                mpl.lines.Line2D([x1_trailer1 - 3. / 8. * dia * st2, x1_trailer1 - 0.5 * dia * st2],
                                 [x2_trailer1 + 3. / 8. * dia * ct2, x2_trailer1 + 0.5 * dia * ct2], color='k',
                                 zorder=1, linewidth=2.0))

            # trailer 2
            trailer2 = mpl.patches.FancyBboxPatch((x1_trailer2 - dia * 0.25, x2_trailer2 - dia * 0.25), 0.5 * dia,
                                                  0.5 * dia, color='0.5', zorder=0, fill=True, mutation_scale=0.05)
            t_start = self.axes.transData
            t = mpl.transforms.Affine2D().rotate_around(x1_trailer2, x2_trailer2, theta3)
            t_end = t + t_start
            trailer2.set_transform(t_end)
            self.axes.add_patch(trailer2)

            wheel_t21 = self.axes.add_line(
                mpl.lines.Line2D([x2T1_1, x2T1_2], [y2T1_1, y2T1_2], color='k', zorder=1, linewidth=3.0))

            wheel_t22 = self.axes.add_line(
                mpl.lines.Line2D([x2T2_1, x2T2_2], [y2T2_1, y2T2_2], color='k', zorder=1, linewidth=3.0))

            axis3 = self.axes.add_line(
                mpl.lines.Line2D([x1_trailer2 + 3. / 8. * dia * st3, x1_trailer2 + 0.5 * dia * st3],
                                 [x2_trailer2 - 3. / 8. * dia * ct3, x2_trailer2 - 0.5 * dia * ct3], color='k',
                                 zorder=1, linewidth=2.0))

            axis4 = self.axes.add_line(
                mpl.lines.Line2D([x1_trailer2 - 3. / 8. * dia * st3, x1_trailer2 - 0.5 * dia * st3],
                                 [x2_trailer2 + 3. / 8. * dia * ct3, x2_trailer2 + 0.5 * dia * ct3], color='k',
                                 zorder=1, linewidth=2.0))

            self.image = [sphere, wheel_1, wheel_2, track, rod_1, rod_2, rod_3, rod_4, joint_1, joint_2, trailer1,
                          trailer2, wheel_t11, wheel_t12, wheel_t21, wheel_t22, axis1, axis2, axis3, axis4]

        else:
            # IPS()
            # chassis
            self.image[0].center = (x1, x2)

            # wheel2
            self.image[1].set_data([xR1_1, xR1_2], [yR1_1, yR1_2])

            # wheel2
            self.image[2].set_data([xR2_1, xR2_2], [yR2_1, yR2_2])
            # track
            # self.image[3].set_data([self.x[:,0]],[self.x[:,1]])
            # flat track
            # self.image[3].set_data([self.P[:,0]],[self.P[:,1]])

            # rod 1
            self.image[4].set_data([x1, x1_joint1], [x2, x2_joint1])

            # rod 2
            self.image[5].set_data([x1_joint1, x1_trailer1], [x2_joint1, x2_trailer1])

            # rod 3
            self.image[6].set_data([x1_trailer1, x1_joint2], [x2_trailer1, x2_joint2])

            # rod 4
            self.image[7].set_data([x1_joint2, x1_trailer2], [x2_joint2, x2_trailer2])

            # joint1
            self.image[8].center = (x1_joint1, x2_joint1)

            # joint2
            self.image[9].center = (x1_joint2, x2_joint2)

            # trailer 1
            self.image[10].remove()
            trailer1 = mpl.patches.FancyBboxPatch((x1_trailer1 - dia * 0.25, x2_trailer1 - dia * 0.25), 0.5 * dia,
                                                  0.5 * dia, color='0.5', zorder=0, fill=True, mutation_scale=0.05)
            t_start = self.axes.transData
            t = mpl.transforms.Affine2D().rotate_around(x1_trailer1, x2_trailer1, theta2)
            t_end = t + t_start
            trailer1.set_transform(t_end)
            self.axes.add_patch(trailer1)
            self.image[10] = trailer1
            # trailer 2
            self.image[11].remove()
            trailer2 = mpl.patches.FancyBboxPatch((x1_trailer2 - dia * 0.25, x2_trailer2 - dia * 0.25), 0.5 * dia,
                                                  0.5 * dia, color='0.5', zorder=0, fill=True, mutation_scale=0.05)
            t_start = self.axes.transData
            t = mpl.transforms.Affine2D().rotate_around(x1_trailer2, x2_trailer2, theta3)
            t_end = t + t_start
            trailer2.set_transform(t_end)
            self.axes.add_patch(trailer2)
            self.image[11] = trailer2
            self.image[12].set_data([x1T1_1, x1T1_2], [y1T1_1, y1T1_2])
            self.image[13].set_data([x1T2_1, x1T2_2], [y1T2_1, y1T2_2])
            self.image[14].set_data([x2T1_1, x2T1_2], [y2T1_1, y2T1_2])
            self.image[15].set_data([x2T2_1, x2T2_2], [y2T2_1, y2T2_2])
            self.image[16].set_data([x1_trailer1 + 3. / 8. * dia * st2, x1_trailer1 + 0.5 * dia * st2],
                                    [x2_trailer1 - 3. / 8. * dia * ct2, x2_trailer1 - 0.5 * dia * ct2])
            self.image[17].set_data([x1_trailer1 - 3. / 8. * dia * st2, x1_trailer1 - 0.5 * dia * st2],
                                    [x2_trailer1 + 3. / 8. * dia * ct2, x2_trailer1 + 0.5 * dia * ct2])
            self.image[18].set_data([x1_trailer2 + 3. / 8. * dia * st3, x1_trailer2 + 0.5 * dia * st3],
                                    [x2_trailer2 - 3. / 8. * dia * ct3, x2_trailer2 - 0.5 * dia * ct3])
            self.image[19].set_data([x1_trailer2 - 3. / 8. * dia * st3, x1_trailer2 - 0.5 * dia * st3],
                                    [x2_trailer2 + 3. / 8. * dia * ct3, x2_trailer2 + 0.5 * dia * ct3])

        self.canvas.draw()

    def calc_positions(self, x1, x2, theta1, theta2, theta3, dia, d1, l2, d2, l3, car_wheelius, wheel, ct1, st1, ct2,
                       st2,
                       ct3, st3):
        # wheel 1
        xR1_1 = x1 + st1 * car_wheelius - ct1 * wheel
        yR1_1 = x2 - ct1 * car_wheelius - st1 * wheel
        xR1_2 = x1 + st1 * car_wheelius + ct1 * wheel
        yR1_2 = x2 - ct1 * car_wheelius + st1 * wheel

        # wheel2
        xR2_1 = x1 - st1 * car_wheelius - ct1 * wheel
        yR2_1 = x2 + ct1 * car_wheelius - st1 * wheel
        xR2_2 = x1 - st1 * car_wheelius + ct1 * wheel
        yR2_2 = x2 + ct1 * car_wheelius + st1 * wheel

        # rod 1
        x1_joint1 = x1 - d1 * ct1
        x2_joint1 = x2 - d1 * st1

        # trailer 1
        x1_trailer1 = x1_joint1 - l2 * ct2
        x2_trailer1 = x2_joint1 - l2 * st2

        # trailer 1 wheel 1
        x1T1_1 = x1_trailer1 + st2 * car_wheelius - ct2 * wheel
        y1T1_1 = x2_trailer1 - ct2 * car_wheelius - st2 * wheel
        x1T1_2 = x1_trailer1 + st2 * car_wheelius + ct2 * wheel
        y1T1_2 = x2_trailer1 - ct2 * car_wheelius + st2 * wheel

        # trailer 1 wheel2
        x1T2_1 = x1_trailer1 - st2 * car_wheelius - ct2 * wheel
        y1T2_1 = x2_trailer1 + ct2 * car_wheelius - st2 * wheel
        x1T2_2 = x1_trailer1 - st2 * car_wheelius + ct2 * wheel
        y1T2_2 = x2_trailer1 + ct2 * car_wheelius + st2 * wheel

        # rod 2
        x1_joint2 = x1_trailer1 - d2 * ct2
        x2_joint2 = x2_trailer1 - d2 * st2

        # trailer 2
        x1_trailer2 = x1_joint2 - l3 * ct3
        x2_trailer2 = x2_joint2 - l3 * st3

        # trailer 2 wheel 1
        x2T1_1 = x1_trailer2 + st3 * car_wheelius - ct3 * wheel
        y2T1_1 = x2_trailer2 - ct3 * car_wheelius - st3 * wheel
        x2T1_2 = x1_trailer2 + st3 * car_wheelius + ct3 * wheel
        y2T1_2 = x2_trailer2 - ct3 * car_wheelius + st3 * wheel

        # trailer  wheel 2
        x2T2_1 = x1_trailer2 - st3 * car_wheelius - ct3 * wheel
        y2T2_1 = x2_trailer2 + ct3 * car_wheelius - st3 * wheel
        x2T2_2 = x1_trailer2 - st3 * car_wheelius + ct3 * wheel
        y2T2_2 = x2_trailer2 + ct3 * car_wheelius + st3 * wheel

        return (xR1_1, yR1_1, xR1_2, yR1_2, xR2_1, yR2_1, xR2_2, yR2_2,
                x1_joint1, x2_joint1,
                x1_trailer1, x2_trailer1,
                x1T1_1, y1T1_1, x1T1_2, y1T1_2, x1T2_1, y1T2_1, x1T2_2, y1T2_2,
                x1_joint2, x2_joint2,
                x1_trailer2, x2_trailer2,
                x2T1_1, y2T1_1, x2T1_2, y2T1_2, x2T2_1, y2T2_1, x2T2_2, y2T2_2)


pm.register_visualizer(CarVisualizer)



"""
test_simulation_interface
----------------------------------

Tests for `simulation_interface` module.
"""

import sys
import unittest
from PyQt5.QtCore import Qt

from pymoskito.simulation_interface import PropertyItem, PropertyDelegate


class TestPropertyItem(unittest.TestCase):

    def setUp(self):
        self.props = {"Name": "TestName",
                      "Integer Value": 12,
                      "Float Value": 3.14,
                      "List": [1, 2],
                      "None": None,
                      "Dictionary": dict(a=2, b=7),
                      }

    def test_data(self):
        # Edit is performed using the raw data
        key_items = []
        val_items = []
        for key, val in self.props.items():
            key_items.append(PropertyItem(key))
            val_items.append(PropertyItem(val))

        for idx in range(len(key_items)):
            # default role should return string
            name = key_items[idx].data(role=Qt.DisplayRole)
            self.assertIsInstance(name, str)

            data_string = val_items[idx].data(role=Qt.DisplayRole)
            self.assertIsInstance(data_string, str)
            self.assertEqual(data_string, str(self.props[name]))

            # Data should be available in raw format using special role
            data = val_items[idx].data(role=PropertyItem.RawDataRole)
            self.assertEqual(self.props[name], data)

    def test_setData(self):
        # only if the returned string can be parsed it should be accepted
        item = PropertyItem(self.props["List"])

        edit_str = item.data(role=Qt.EditRole)
        self.assertIsInstance(edit_str, str)
        self.assertEqual(edit_str, str(self.props["List"]))

        new_list = [1, 3, 5]
        item.setData(new_list, role=PropertyItem.RawDataRole)
        self.assertEqual(item.data(role=PropertyItem.RawDataRole), new_list)
        new_str = str(new_list)
        self.assertEqual(item.data(role=Qt.DisplayRole), new_str)

        # invalid data should be rejected and exception should be printed to log
        item.setData(edit_str[:-1], role=Qt.EditRole)
        self.assertEqual(item.data(role=Qt.DisplayRole), new_str)



# -*- coding: utf-8 -*-
import pickle
import logging
import os

from PyQt5.QtCore import Qt, pyqtSignal, QSize, QSettings
from PyQt5.QtGui import QIcon, QKeySequence
from PyQt5.QtWidgets import (
    QWidget, QAction, QMainWindow, QListWidget, QListWidgetItem, QToolBar,
    QStatusBar, QLabel, QShortcut, QFileDialog, QGridLayout, QSizePolicy,
    QTextEdit
)

from . import registry as pm
from .processing_core import PostProcessingModule, MetaProcessingModule
from .tools import get_resource, PlainTextLogger
from .mpl_settings import enable_latex, disable_latex

__all__ = ["PostProcessor"]


class PostProcessor(QMainWindow):

    sim_results_changed = pyqtSignal()
    post_results_changed = pyqtSignal()

    figures_changed = pyqtSignal(list, str)

    def __init__(self, settings=None, parent=None):
        QMainWindow.__init__(self, parent)
        self._logger = logging.getLogger(self.__class__.__name__)
        self._settings = settings
        if settings is None:
            self._settings = QSettings()

        self.setWindowTitle("Processing")
        self.setWindowIcon(QIcon(get_resource("processing.png")))
        self.mainFrame = QWidget(self)
        self.resize(1000, 600)

        # toolbar
        self.toolBar = QToolBar("file control")
        self.toolBar.setIconSize(QSize(24, 24))
        self.addToolBar(self.toolBar)

        self.actLoad = QAction(self)
        self.actLoad.setText("load result file")
        self.actLoad.setIcon(QIcon(get_resource("load.png")))
        self.actLoad.setDisabled(False)
        self.actLoad.triggered.connect(self.load_result_files)

        self.actPostLoad = QAction(self)
        self.actPostLoad.setText("load post-result file")
        self.actPostLoad.setIcon(QIcon(get_resource("load.png")))
        self.actPostLoad.setDisabled(False)
        self.actPostLoad.triggered.connect(self.load_post_result_files)

        self.actSwitch = QAction(self)
        self.actSwitch.setText("switch display mode")
        self.actSwitch.setIcon(QIcon(get_resource("left_mode.png")))
        self.actSwitch.setDisabled(False)
        self.actSwitch.triggered.connect(self.switch_sides)
        self.displayLeft = True

        self.spacer1 = QWidget()
        self.spacer2 = QWidget()
        self.spacer1.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.spacer2.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.actReloadMethods = QAction(self)
        self.actReloadMethods.setText("reload methods")
        self.actReloadMethods.setIcon(QIcon(get_resource("reload.png")))
        self.actReloadMethods.setDisabled(False)
        self.actReloadMethods.triggered.connect(self.update_post_method_list)

        self.actReloadMetaMethods = QAction(self)
        self.actReloadMetaMethods.setText("reload meta methods")
        self.actReloadMetaMethods.setIcon(QIcon(get_resource("reload.png")))
        self.actReloadMetaMethods.setDisabled(False)
        self.actReloadMetaMethods.triggered.connect(
            self.update_meta_method_list)
        
        self.toolBar.addAction(self.actLoad)
        self.toolBar.addAction(self.actReloadMethods)
        
        self.toolBar.addWidget(self.spacer1)
        self.toolBar.addAction(self.actSwitch)
        self.toolBar.addWidget(self.spacer2)

        self.toolBar.addAction(self.actReloadMetaMethods)
        self.toolBar.addAction(self.actPostLoad)

        # main window
        self.grid = QGridLayout(self.mainFrame)
        self.grid.setColumnMinimumWidth(0, 70)
        self.grid.setColumnStretch(0, 0)
        self.grid.setColumnStretch(1, 1)

        self.methodList = QListWidget(self)
        self.methodList.itemDoubleClicked.connect(
            self.post_processor_clicked)
        self.update_post_method_list()
        self.metaMethodList = QListWidget(self)
        self.metaMethodList.itemDoubleClicked.connect(
            self.meta_processor_clicked)
        self.update_meta_method_list()

        self.sim_result_list = QListWidget(self)
        self.sim_results_changed.connect(self.update_result_list)
        self.results = []

        self.delShort = QShortcut(QKeySequence(Qt.Key_Delete),
                                  self.sim_result_list)
        self.delShort.activated.connect(self.remove_result_item)

        # figures
        self._figure_dict = {}
        self.figures_changed.connect(self.update_figure_lists)

        self.post_figure_list = QListWidget(self)
        self.post_figure_list.currentItemChanged.connect(
            self.current_figure_changed)
        self.meta_figure_list = QListWidget(self)
        self.meta_figure_list.currentItemChanged.connect(
            self.current_figure_changed)

        self.plotView = QWidget()
        self.lastFigure = None

        self.post_result_list = QListWidget(self)
        self.post_results_changed.connect(self.update_post_result_list)
        self.post_results = []
        self.delShortPost = QShortcut(QKeySequence(Qt.Key_Backspace),
                                      self.post_result_list)
        self.delShortPost.activated.connect(self.remove_post_result_item)

        # log dock
        self.logBox = QTextEdit(self)
        self.logBox.setReadOnly(True)
        self.logBox.setLineWrapMode(QTextEdit.NoWrap)
        self.logBox.ensureCursorVisible()

        self.grid.addWidget(QLabel("Result Files:"), 0, 0)
        self.grid.addWidget(self.sim_result_list, 1, 0)
        self.grid.addWidget(QLabel("Postprocessors:"), 2, 0)
        self.grid.addWidget(self.methodList, 3, 0)
        self.grid.addWidget(QLabel("Figures:"), 4, 0)
        self.grid.addWidget(self.post_figure_list, 5, 0)
        self.grid.addWidget(QLabel("Selected Figure:"), 0, 1)
        self.grid.addWidget(QLabel("Postprocessor Files:"), 0, 2)
        self.grid.addWidget(self.post_result_list, 1, 2)
        self.grid.addWidget(QLabel("Metaprocessors:"), 2, 2)
        self.grid.addWidget(self.metaMethodList, 3, 2)
        self.grid.addWidget(QLabel("Figures:"), 4, 2)
        self.grid.addWidget(self.meta_figure_list, 5, 2)
        self.grid.addWidget(self.logBox, 6, 0, 1, 3)

        self.mainFrame.setLayout(self.grid)
        self.setCentralWidget(self.mainFrame)

        # init logger for logging box
        self.textLogger = PlainTextLogger(self._settings, logging.INFO)
        self.textLogger.set_target_cb(self.logBox)
        logging.getLogger().addHandler(self.textLogger)

        # status bar
        self.statusBar = QStatusBar(self)
        self.setStatusBar(self.statusBar)

        # activate latex formatting in plots
        self._logger.info("TeX on")
        enable_latex()

    def closeEvent(self, QCloseEvent):
        self._logger.info("Close Event received, closing Postprocessing GUI")
        self._logger.info("TeX off")
        disable_latex()
        super().closeEvent(QCloseEvent)

    def load_result_files(self):
        path = self._settings.value("path/simulation_results")

        dialog = QFileDialog(self)
        dialog.setFileMode(QFileDialog.ExistingFiles)
        dialog.setDirectory(path)
        dialog.setNameFilter("PyMoskito Result files (*.pmr)")

        if dialog.exec_():
            files = dialog.selectedFiles()
            for single_file in files:
                if single_file:
                    self._load_result_file(single_file)

    def _load_result_file(self, file_name):
        """
        loads a result file
        """
        self._logger.info("loading result file {}".format(file_name))
        with open(file_name.encode(), "rb") as f:
            self.results.append(pickle.load(f))

        self.sim_results_changed.emit()

    def update_result_list(self):
        self.sim_result_list.clear()
        for res in self.results:
            name = res["regime name"]
            self.sim_result_list.addItem(name)

    def remove_result_item(self):
        if self.sim_result_list.currentRow() >= 0:
            del self.results[self.sim_result_list.currentRow()]
            self.sim_result_list.takeItem(self.sim_result_list.currentRow())

    def load_post_result_files(self):
        path = self._settings.value("path/processing_results")

        dialog = QFileDialog(self)
        dialog.setFileMode(QFileDialog.ExistingFiles)
        dialog.setDirectory(path)
        dialog.setNameFilter("Postprocessing Output files (*.pof)")

        if dialog.exec_():
            files = dialog.selectedFiles()
            for single_file in files:
                if single_file:
                    self._load_post_result_file(single_file)

    def _load_post_result_file(self, file_name):
        """
        loads a post-result file (.pof)
        """
        name = os.path.split(file_name)[-1][:-4]
        self._logger.info("loading result file {}".format(file_name))
        with open(file_name.encode(), "rb") as f:
            results = pickle.load(f)
            results.update({"name": name})
            self.post_results.append(results)

        self.post_results_changed.emit()

    def update_post_result_list(self):
        self.post_result_list.clear()
        for res in self.post_results:
            name = res["name"]
            self.post_result_list.addItem(name)

    def remove_post_result_item(self):
        if self.post_result_list.currentRow() >= 0:
            del self.post_results[self.post_result_list.currentRow()]
            self.post_result_list.takeItem(self.post_result_list.currentRow())

    def update_post_method_list(self):
        self.methodList.clear()
        modules = pm.get_registered_processing_modules(PostProcessingModule)
        for mod in modules:
            self.methodList.addItem(mod[1])

    def update_meta_method_list(self):
        self.metaMethodList.clear()
        modules = pm.get_registered_processing_modules(MetaProcessingModule)
        for mod in modules:
            self.metaMethodList.addItem(mod[1])

    def post_processor_clicked(self, item):
        self.run_processor(str(item.text()), "post")

    def meta_processor_clicked(self, item):
        self.run_processor(str(item.text()), "meta")

    def run_processor(self, name, processor_type):
        if processor_type == "post":
            result_files = self.results
            base_cls = PostProcessingModule
        elif processor_type == "meta":
            result_files = self.post_results
            base_cls = MetaProcessingModule
        else:
            self._logger.error("unknown processor type {0}".format(
                processor_type))
            raise ValueError("unknown processor type {0}".format(
                processor_type))

        if not result_files:
            self._logger.warning("run_processor() Error: no result file loaded")
            return

        processor_cls = pm.get_processing_module_class_by_name(base_cls, name)
        processor = processor_cls()

        figs = []
        try:
            self._logger.info("executing processor '{0}'".format(name))
            figs = processor.process(result_files)
        except Exception as err:
            self._logger.error("Error in processor")
            self._logger.exception(err)

        self.figures_changed.emit(figs, processor_type)
        self._logger.info("finished postprocessing")

    def update_figure_lists(self, figures, target_type):
        # remove no longer needed elements
        for item, fig in [(key, val[0])
                          for key, val in self._figure_dict.items()
                          if val[1] == target_type]:
            if fig not in [new_fig["figure"] for new_fig in figures]:
                if target_type == "post":
                    old_item = self.post_figure_list.takeItem(
                        self.post_figure_list.row(item))
                    del old_item
                elif target_type == "meta":
                    old_item = self.meta_figure_list.takeItem(
                        self.meta_figure_list.row(item))
                    del old_item

                del self._figure_dict[item]

        # add new ones to internal storage
        for fig in figures:
            if fig["figure"] not in self._figure_dict.values():
                new_entry = [(fig["name"],
                              (QListWidgetItem(fig["name"]),
                               fig["figure"], target_type)
                              )]
                self._figure_dict.update(new_entry)

        # add to display
        for key, val in self._figure_dict.items():
            if val[2] == "post":
                self.post_figure_list.addItem(val[0])
            elif val[2] == "meta":
                self.meta_figure_list.addItem(val[0])

        self.post_figure_list.setCurrentItem(self.post_figure_list.item(0))
        self.meta_figure_list.setCurrentItem(self.meta_figure_list.item(0))

    def current_figure_changed(self, current_item, last_item=None):
        if current_item is None:
            return

        figures = self._figure_dict

        if self.lastFigure:
            self.grid.removeWidget(self.lastFigure)
            self.lastFigure.setVisible(False)

        if current_item.text() in figures:
            figure_widget = figures[current_item.text()][1]
            self.grid.addWidget(figure_widget, 1, 1, 5, 1)
            figure_widget.setVisible(True)
            self.lastFigure = figure_widget
        
    def switch_sides(self):
        self.displayLeft = not self.displayLeft
        if self.displayLeft:
            self.actSwitch.setIcon(QIcon(get_resource("left_mode.png")))
            self.post_figure_list.setFocus()
            self.current_figure_changed(self.post_figure_list.currentItem())
        else:
            self.actSwitch.setIcon(QIcon(get_resource("right_mode.png")))
            self.meta_figure_list.setFocus()
            self.current_figure_changed(self.meta_figure_list.currentItem())



import numpy as np
from collections import OrderedDict

import pymoskito as pm

from . import settings as st


#class
class BallBeamModel(pm.Model):
    """
    Implementation of the Ball and Beam System
    """
    public_settings = OrderedDict([("M", st.M),
                                   ("R", st.R),
                                   ("J", st.J),
                                   ("Jb", st.Jb),
                                   ("G", st.G),
                                   ("beam length", st.beam_length),
                                   ("beam width", st.beam_width),
                                   ("beam depth", st.beam_depth),
                                   ("initial state", st.initial_state)
                                   ])

    #init
    def __init__(self, settings):
        # add specific "private" settings
        settings.update(state_count=4)
        settings.update(input_count=1)
        settings.update({"output_info": {
            0: {"Name": "ball position", "Unit": "m"},
            2: {"Name": "beam angle", "Unit": "rad"},
        }})
        pm.Model.__init__(self, settings)

        # shortcuts for readability
        self.M = self._settings['M']
        self.R = self._settings['R']
        self.J = self._settings['J']
        self.Jb = self._settings['Jb']
        self.G = self._settings['G']
        self.B = self.M / (self.Jb / self.R ** 2 + self.M)

    #state
    def state_function(self, t, x, args):
        """
        Calculations of system state changes
        :param x: state
        :param t: time
        :type args: system input tau
        """

        # definitional
        x1 = x[0]
        x2 = x[1]
        x3 = x[2]
        x4 = x[3]
        tau = args[0].squeeze()

        dx1 = x2
        dx2 = self.B * (x1 * x4 ** 2 - self.G * np.sin(x3))
        dx3 = x4

        # inverse nonlinear system transformation
        u = (tau - self.M * (2 * x1 * x2 * x4
                             + self.G * x1 * np.cos(x3))) / (self.M * x1 ** 2
                                                             + self.J + self.Jb)
        dx4 = u

        return np.array([dx1, dx2, dx3, dx4])

    #root
    def root_function(self, x):
        """
        is not used
        :param x: state
        :return:
        """
        return [False]

    #consistency
    def check_consistency(self, x):
        """
        Check if the ball remains on the beam
        :param x: state
        """
        if abs(x[0]) > float(self._settings['beam length']) / 2:
            raise pm.ModelException('Ball fell down.')
        if abs(x[2]) > np.pi / 2:
            raise pm.ModelException('Beam reached critical angle.')

    #output
    def calc_output(self, input_vector):
        """
        return ball position as output
        :param input_vector: input values
        :return: ball position
        """
        return input_vector[0]


#register
pm.register_simulation_module(pm.Model, BallBeamModel)



# -*- coding: utf-8 -*-

"""
test_tools
----------

Tests for several tools and functions
"""

import unittest
import numpy as np
import sympy as sp

import pymoskito as pm


class TestControlTools(unittest.TestCase):

    def setUp(self):
        self.A = np.array([[0, 1, 0], [0, 0, 2/25], [0, -800, -2]])
        self.B = np.array([[0], [0], [100]])
        self.C = np.array([[1, 0, 0]])
        self.Qc = np.array([[0, 0, 8], [0, 8, -16], [100, -200, -6000]])
        self.Qo = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 2/25]])
        self.poles = [-1, -1+1j, -1-1j]
        self.coefficients = np.array([2, 4, 3])
        self.K = np.array([[0.25, -7.5, 0.01]])
        self.L = np.array([[1], [-62], [775]])
        self.V = np.array([[0.25]])
        self.n = 3
        self.m = 1
        self.r = 1

    def test_controlability_matrix(self):
        """
        check the function, which calculates the controlability matrix
        """

        # enter matrices with wrong dimensions
        self.assertRaises(ValueError,
                          pm.controllability_matrix,
                          np.array([[1, 1, 1], [1, 1, 1]]),
                          np.array([[1], [1], [1]]))
        self.assertRaises(ValueError,
                          pm.controllability_matrix,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1], [1], [1]]))
        self.assertRaises(ValueError,
                          pm.controllability_matrix,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1, 1, 1], [1, 1, 1]]))

        # enter matrices, where the system shall be not controllable
        self.assertRaises(ValueError,
                          pm.controllability_matrix,
                          np.array([[0, 0], [0, 0]]),
                          np.array([[0], [0]]))

        # calculate an example
        test_Qc = pm.controllability_matrix(self.A, self.B)
        self.assertTrue(np.allclose(test_Qc, self.Qc))
        self.assertTrue(np.allclose(
            pm.controllability_matrix(self.A.tolist(), self.B.tolist()),
            self.Qc))

        # check dimensions
        self.assertEqual(test_Qc.shape, (self.n, self.n*self.m))

    def test_observability_matrix(self):
        """
        check the function, which calculates the observability matrix
        """

        # enter matrices with wrong dimensions
        self.assertRaises(ValueError,
                          pm.observability_matrix,
                          np.array([[1, 1, 1], [1, 1, 1]]),
                          np.array([[1, 1, 1]]))
        self.assertRaises(ValueError,
                          pm.observability_matrix,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1, 1, 1]]))
        self.assertRaises(ValueError,
                          pm.observability_matrix,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1, 1], [1, 1], [1, 1]]))

        # enter matrices, where the system shall be not observable
        self.assertRaises(ValueError,
                          pm.observability_matrix,
                          np.array([[0, 0], [0, 0]]),
                          np.array([[0, 0]]))

        # calculate an example
        test_Qo = pm.observability_matrix(self.A, self.C)
        self.assertTrue(np.allclose(test_Qo, self.Qo))
        self.assertTrue(np.allclose(
            pm.observability_matrix(self.A.tolist(), self.C.tolist()),
            self.Qo))

        # check dimensions
        self.assertEqual(test_Qo.shape, (self.r*self.n, self.n))

    def test_get_coefficients(self):
        """
        check function, which calculates the coefficients of a characteristic
        polynomial
        """

        # calculate an example
        co = pm.char_coefficients(self.poles)
        self.assertTrue(np.allclose(co, self.coefficients))
        self.assertEqual(co.size, self.coefficients.size)

    def test_place_siso(self):
        """
        test for pole placement
        """

        # ValueError must occur if it is a MIMO system
        self.assertRaises(ValueError, pm.place_siso,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1, 1], [1, 1]]),
                          self.poles)
        self.assertRaises(ValueError, pm.place_siso,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1], [1]]),
                          np.array([1, 1, 1]))

        # calculate a state feedback and an observer gain
        test_K = pm.place_siso(self.A, self.B, self.poles)
        test_L = pm.place_siso(self.A.transpose(),
                                                   self.C.transpose(),
                                                   self.poles).transpose()

        self.assertTrue(np.allclose(test_K, self.K))
        self.assertTrue(np.allclose(test_L, self.L))

    def test_prefilter(self):
        """
        check function, which calculates the prefilter matrix
        """

        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[1], [1]]),
                          np.array([[1], [1]]),
                          np.array([[1, 1]]))
        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1], [1], [1]]),
                          np.array([[1, 1]]))
        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1, 1, 1], [1, 1, 1]]),
                          np.array([[1, 1]]))

        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1], [1]]),
                          np.array([[1, 1, 1]]))
        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[1, 1], [1, 1]]),
                          np.array([[1], [1]]),
                          np.array([[1, 1], [1, 1], [1, 1]]))

        # enter matrices, where the prefilter's calculation should fail
        self.assertRaises(ValueError, pm.calc_prefilter,
                          np.array([[0, 0], [0, 0]]), np.array([[0], [0]]),
                          np.array([[0, 0]]), np.array([[0, 0]]))

        test_V = pm.calc_prefilter(self.A, self.B, self.C, self.K)
        self.assertTrue(np.allclose(test_V, self.V))


class TestLieMath(unittest.TestCase):

    def setUp(self):
        self._x1, self._x2 = sp.symbols("x y")
        self.x = sp.Matrix([self._x1, self._x2])
        self.f = sp.Matrix([-self._x2**2, sp.sin(self._x1)])
        self.h = sp.Matrix([self._x1**2 - sp.sin(self._x2)])

    def test_lie_derivative(self):
        Lfh = pm.lie_derivatives(self.h, self.f, self.x, 0)
        self.assertEqual(Lfh, [self.h])

        Lfh = pm.lie_derivatives(self.h, self.f, self.x, 1)
        self.assertEqual(Lfh, [self.h,
                               sp.Matrix([-2*self._x1*self._x2**2
                                          - sp.sin(self._x1)*sp.cos(self._x2)])
                               ])
        Lfh = pm.lie_derivatives(self.h, self.f, self.x, 2)
        self.assertEqual(Lfh, [self.h,
                               sp.Matrix([-2*self._x1*self._x2**2
                                          - sp.sin(self._x1)*sp.cos(self._x2)]),
                               sp.Matrix([
                                   -self._x2**2*(
                                       -2*self._x2**2
                                       - sp.cos(self._x1)*sp.cos(self._x2)
                                   )
                                   + sp.sin(self._x1)*(
                                       - 4*self._x1*self._x2
                                       + sp.sin(self._x1)*sp.sin(self._x2)
                                   )])
                               ])

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
This file contains default settings for physical simulation of the ball and beam
system. These values are used to maintain consistent settings over the whole
test system. The values given here are used to generate regime lists that are
used to perform experimental studies.
"""

# standard values for integration
step_size = 0.0005  # to produce same error dimension as hauser
measure_rate = 1000  # to produce same error dimension as hauser

# initial minimal state vector (r, dr, theta, dtheta)
# default values (used if none is given)
initial_state = [0, 0, 0, 0]

# integrator defaults
int_mode = "vode"
int_method = "adams"
int_rtol = 1e-6
int_atol = 1e-9
end_time = 20

# nominal system values
M = 0.05  # kg
R = 0.01  # m
J = 0.02  # kgm^2
Jb = 2e-6  # kgm^2
G = 9.81  # m/s^2
B = M / (Jb / R ** 2 + M)
beam_width = 0.2  # m
beam_depth = 0.4  # m
beam_length = 9  # m

# operating point
q_op = [0, 0, 0, 0]
tau_op = 0

# poles for LSSController
poles_LSSController = [-2, -2, -2, -2]

# poles for LuenbergerObserver
poles_LuenbergerObserver = [-3, -3, -3, -3]

# Visualization
ballScale = 30
visR = R * ballScale
visBeamWidth = beam_width
visBeamLength = beam_length
visBeamDepth = beam_depth

# mpl settings
x_min_plot = -.6 * visBeamLength
x_max_plot = .6 * visBeamLength
y_min_plot = -(visR + visBeamLength / 3 + visBeamWidth)
y_max_plot = visR + visBeamLength / 3

# -----------------------------------------------
# export settings latex-style
# -----------------------------------------------
latex_font_size = 14
label_size = 1 * latex_font_size
title_size = 1.5 * latex_font_size

# -----------------------------------------------
# color-settings for diagramms with more than
# one controller
# -----------------------------------------------
color_cycle = {'FController': 'royalblue',
               'GController': 'indianred',
               'JController': 'mediumorchid',
               'LSSController': 'limegreen',
               'PIFeedbackController': 'darkorange',
               }

# -----------------------------------------------
# TestCase A1 - Pole Variation
# -----------------------------------------------
paramPoleVariation = {
    'FController': {
        'pole_limits': [0, -3.5],
        'pole_step': 0.1,
    },
    'GController': {
        'pole_limits': [0, -3.3],
        'pole_step': 0.1,
    },
    # J FAILS
    'JController': {
        'pole_limits': [0, -4],
        'pole_step': 0.1,
    },
    'LSSController': {
        'pole_limits': [0, -3.9],
        'pole_step': 0.1,
    },
    'PIFeedbackController': {
        'pole_limits': [0, -4],
        'pole_step': 0.1,
    },
}

# -----------------------------------------------
# TestCase A - Step and other Responses
# -----------------------------------------------

# best konwon poles - gained in A1
# JController is for a step for all tested poles unstable!!!!!
# To give the JController a chance it is tested for poles [-0.5, ca. 0.1, -15]
# on the SmoothTransitionTrajectory (Position:[0,3], delta t:5)
poles = {'FController': -3.1,
         'GController': -2.8,
         'JController': -2.0,
         # -2.0 hauser standard
         'LSSController': -3.4,
         'PIFeedbackController': -3.7,
         }

# -----------------------------------------------
# TestCase B - parameter variation
# -----------------------------------------------
# parameter variation list
paramVariationListB = ['M', 'Jb']
paramToleranceList = [0.2, 0.3]

# parameter stability limits for best known poles
paramStabilityLimits = {
    'FController': {
        'M': [0.01, 1.0],
        'Jb': [0, 10e-6],
    },
    'GController': {
        'M': [0.01, 1],
        'Jb': [0, 10e-6],
    },
    # J FAILS
    'JController': {
        'M': [0.05, 10],
        'Jb': [0, 10e-6],
    },
    'LSSController': {
        'M': [0.01, 1],
        'Jb': [0, 10e-6],
    },
    'PIFeedbackController': {
        'M': [0.01, 1],
        'Jb': [0, 10e-6],
    },
}

# -----------------------------------------------
# TestCase C1 - parameter variation
# -----------------------------------------------
# parameter stability limits (plus a little bit)
# for sigma and delay for best known poles
paramStabilityLimitsSigmaDelay = {
    'FController': {
        'sigma': [0, 0.45],
        'sigma_step': 0.01,
        'delay': [0, 85],
        'delay_step': 5,
        },
    'GController': {
        'sigma': [0, 0.35],
        'sigma_step': 0.01,
        'delay': [0, 85],
        'delay_step': 5,
        },
    # J FAILS
    'JController': {
        'sigma': [0, 0],
        'sigma_step': 0.1,
        'delay': [0, 0],
        'delay_step': 1,
        },
    'LSSController': {
        'sigma': [0, 1],
        'sigma_step': 0.05,
        'delay': [0, 4],
        'delay_step': 1,
        },
    'PIFeedbackController': {
        'sigma': [0, 0.45],
        'sigma_step': 0.01,
        'delay': [0, 8],
        'delay_step': 1,
        },
    }

# -----------------------------------------------
# TestCase C2 - Limiter Variation
# -----------------------------------------------
smoothPoles = {'FController': -2.0,
               'GController': -2.0,
               'JController': -2.0,
               'LSSController': -3.4,
               'PIFeedbackController': -1.7,
               }
paramVariationDictC2 = {'limiter': {'lower_bound': round(M * G * 4.5, 1),
                                    'upper_bound': 3,
                                    'step_size': 0.1,
                                    }}



# -*- coding: utf-8 -*-
import os
import matplotlib as mpl
import numpy as np
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from matplotlib.lines import Line2D

import pymoskito as pm

from . import settings as st


class BallBeamStepResponse(pm.StepResponse):
    """
    Postprocessor the creates Step Response Diagrams for the Ballbeam system.

    Run the `pp_regimes.sreg` file to create the required simulation data.
    """

    def __init__(self):
        super().__init__(model_idx=0, trajectory_idx=0)


pm.register_processing_module(pm.PostProcessingModule, BallBeamStepResponse)


# TODO get those working again
class EvalA1(pm.PostProcessingModule):
    """
    create diagrams for evaluation step A1
    """
    name = 'A1'

    line_color = '#aaaaaa'
    line_style = '-'
    font_size = 20
    #    epsPercent = 2./5
    spacing = 0.01
    counter = 0

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        self.label_positions = None

    def calc_metrics(self, data, output):
        """
        calculate metrics for comaprism
        """

        L1NormITAE = self.calcL1NormITAE(data)
        L1NormAbs = self.calcL1NormAbs(data)
        #
        #        print 'ITAE score: ', errorIntegral
        print('L1NormITAE: ', L1NormITAE)
        print('L1NormAbs: ', L1NormAbs)
        print('\n')
        output.update({'L1NormITAE': L1NormITAE, 'L1NormAbs': L1NormAbs})

    def create_time_line(self, axes, t, y, time_value, label):
        """
        creates a  vertical line in the diagram, reaching from the x-axis to the plot at a given time t

        :param axes:
        :param t:
        :param y:
        :param time_value:
        :param label:
        :returns new counter value
        """
        # don't create lines on the very left
        if time_value == t[-1]:
            return

        # create timeLine
        time_line = Line2D([time_value, time_value],
                           [np.min(y), y[t.index(time_value)]],
                           ls=self.line_style,
                           c=self.line_color)
        axes.add_line(time_line)
        axes.text(time_value + self.spacing, self.label_posistions[self.label_counter], label, size=self.font_size)

    def run(self, data):
        self.label_counter = 0

        # map data-sets
        t = data["results"]["time"]
        y = data["results"]["Solver"][:, 0]
        yd = data["results"]["Trajectory"][-1, 0, 0]

        # evenly distribute labels
        self.label_positions = np.arange(np.min(y) + 0.1 * yd, yd, (yd - np.min(y)) / 4)

        # create plot
        self.axes.set_title(r"\textbf{Sprungantwort}")
        self.axes.plot(t, y, c="k")
        self.axes.set_xlim(left=0, right=t[-1])
        self.axes.set_ylim(0, 3.5)
        self.axes.set_xlabel(r"\textit{Zeit [s]}")
        self.axes.set_ylabel(r"\textit{Ballposition r(t) [m]}")

        # create desired line
        desired_line = Line2D([0, t[-1]], [yd, yd], lw=1, ls=self.line_style, c='k')
        self.axes.add_line(desired_line)

        return

        # calc rise-time (Anstiegszeit)
        try:
            tr = t[y.index([x for x in y if x > yd * 0.9][0])]
            self.create_time_line(self.axes, t, y, tr, r"$T_r$")
            self.results.update({"tr": tr})
        except IndexError:
            self._logger.warning("failed to calculate rise-time 't_r'")
            self.results.update({"tr": None})

        # calc correction-time (Anregelzeit)
        try:
            t_corr = t[y.index([x for x in y if x > yd][0])]
            self.create_time_line(self.axes, t, y, t_corr, r'$T_{anr}$')
            self.results.update({'tanr': t_corr})
        except IndexError:
            self._logger.warning("failed to calculate correction-time 't_corr'")
            self.results.update({'tanr': None})

        # calc overshoot-time and overshoot in percent (Überschwingzeit und Überschwingen)
        if self.results['tanr']:
            if yd > 0:
                y_max = np.max(y[t.index(t_corr):])
            else:
                y_max = np.min(y[t.index(t_corr):])

            t_over = t[y.index(y_max)]
            do = y_max - yd
            do_percent = do / yd * 100
            self.create_time_line(self.axes, t, y, t_over, r'$T_o$')
            self.results.update({'to': t_over, 'do': do, 'doPercent': do_percent})
        else:
            self._logger.warning("failed to calculate overshoot and overshoot time 't_over'")
            self.results.update({'to': None, 'do': None, 'doPercent': None})

        # calc damping-time (Beruhigungszeit)
        try:
            #            eps = self.epsPercent*yd/100
            # TODO relocate settings
            # eps = st.R
            eps = .1
            enter_idx = -1
            for idx, val in enumerate(y):
                if enter_idx == -1:
                    if abs(val - yd) < eps:
                        enter_idx = idx
                else:
                    if abs(val - yd) >= eps:
                        enter_idx = -1
            teps = t[enter_idx]
            # create and add line
            self.create_time_line(self.axes, t, y, teps, r'$T_{\epsilon}$')
            self.results.update({'teps': teps})
        except IndexError:
            # print 'DampingLine is not defined'
            self.results.update({'teps': None})

        # create epsilon tube
        upper_bound = Line2D([0, t[-1]], [yd + eps, yd + eps], ls='--', c=self.line_color)
        self.axes.add_line(upper_bound)
        lower_bound = Line2D([0, t[-1]], [yd - eps, yd - eps], ls='--', c=self.line_color)
        self.axes.add_line(lower_bound)

        # calc control deviation
        control_deviation = y[-1] - yd
        self.results.update({'control_deviation': control_deviation})


class eval_A1_steps_in_one_plot(pm.PostProcessingModule):
    """
    create several step respond in one plot
    """

    line_color = '#aaaaaa'
    line_style = '-'
    font_size = 20
    epsPercent = 2. / 5
    spacing = 0.01
    counter = 0

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def process(self, files):

        print('processing ', self.name)

        # reset counter
        self.counter = 0

        # dict for calculated values
        #        output = {}
        yd = 0

        # create plot
        fig = Figure()
        axes = fig.add_subplot(111)
        axes.set_title(r'Sprungantworten')
        # search t max
        t_max = 0
        for data in files:
            t = data['modules']['solver']['end time']
            if t > t_max:
                t_max = t
        axes.set_xlim(left=0, right=t_max)
        axes.set_ylim(0, 4, 5)
        axes.set_xlabel(r'\textit{Zeit [s]}')
        axes.set_ylabel(r'\textit{Ballposition r(t) [m]}')

        controllerName = []
        # create plots
        counter = 0
        for data in files:
            #            print data['modules']
            #            print data['regime name']
            #            print data['results']['finished']
            # calculate data-sets
            t = data['results']['simTime']
            y = data['results']['model_output.0']
            yd = data['results']['trajectory_output.0'][-1]
            controllerName.append(data['modules']['controller']['type'])
            axes.plot(t, y, label=r'$r_{0} = ' + str(data['modules']['controller']['r0']) + '$',
                      c=mpl.rcParams['axes.color_cycle'][counter])
            counter += 1




            # check for sim success
        #            if not data['results']['finished']:
        #                for key in output.keys():
        #                    output[key] = None

        #        self.posLabel = np.arange(np.min(y) + 0.1*yd, yd, (yd-np.min(y))/4)

        # plot legend
        axes.legend()

        # create desired line
        desiredLine = Line2D([0, t[-1]], [yd, yd], lw=1, ls=self.line_style, c='k')
        axes.add_line(desiredLine)

        # create epsilon tube
        upperBoundLine = Line2D([0, t[-1]], [yd + st.R, yd + st.R], ls='--', c=self.line_color)
        axes.add_line(upperBoundLine)
        lowerBoundLine = Line2D([0, t[-1]], [yd - st.R, yd - st.R], ls='--', c=self.line_color)
        axes.add_line(lowerBoundLine)

        #        #calc control deviation
        #        control_deviation = y[-1] - yd
        #        output.update({'control_deviation': control_deviation})

        #        self.calcMetrics(data, output)


        # add settings and metrics to dictionary results
        results = {}
        #        results.update({'metrics': output})
        #        results.update({'modules': data['modules']})

        canvas = FigureCanvas(fig)

        self.writeOutputFiles(self.name, 'steps_in_one_plot', fig, results)

        return [{'name': '_'.join([controllerName[0], self.name]),
                 'figure': canvas}]

    def calcMetrics(self, data, output):
        '''
        calculate metrics for comaprism
        '''

        L1NormITAE = self.calcL1NormITAE(data)
        L1NormAbs = self.calcL1NormAbs(data)
        #                    
        #        print 'ITAE score: ', errorIntegral
        print('L1NormITAE: ', L1NormITAE)
        print('L1NormAbs: ', L1NormAbs)
        output.update({'L1NormITAE': L1NormITAE, 'L1NormAbs': L1NormAbs})


class EvalA2(pm.PostProcessingModule):
    """
    create diagrams like hauser did
    """
    line_color = '#aaaaaa'
    line_style = '-'

    name = 'A2'

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def run(self, data):
        print('processing ', data['regime name'])

        output = {}

        # calculate data-sets
        t = data['results']['simTime']
        yd = data['results']['trajectory_output.0']
        y = []
        for i in range(4):
            y.append(data['results']['model_output.' + str(i)])

        eps = np.subtract(y[0], yd)
        traj = data['results']['trajectory_output.0']

        # plots
        fig = Figure()
        fig.subplots_adjust(wspace=0.3, hspace=0.25)
        fig.suptitle(r'' + data['modules']['controller']['type'] + ' ' + \
                     '$A_d$ =' + str(data['modules']['trajectory']['Amplitude']) + ' ' + \
                     '$f_d$ =' + str(data['modules']['trajectory']['Frequency']),
                     size=st.title_size)

        axes1 = fig.add_subplot(2, 1, 1)
        # axes2.set_title(r'\textbf{Beam Angle}')
        axes1.plot(t, traj, c='k', ls='-', label='yd(t)')
        axes1.plot(t, y[0], c=st.color_cycle[data['modules']['controller']['type']],
                   ls='-', label='y(t)')
        axes1.set_xlim(left=0, right=t[-1])
        axes1.set_xlabel(r'$t [s]$', size=st.label_size)
        axes1.set_ylabel(r'$r [m]$', size=st.label_size)
        leg = [r'$y_d(t)$', r'$y(t)$']
        axes1.legend(leg, loc=0)

        axes2 = fig.add_subplot(2, 1, 2)
        #        axes.set_title(r'output error = yd - x0')        
        #        deltaE = 0.01
        #        eMax = line([0, t[-1]], [deltaE, deltaE], lw=1,\
        #                            ls='--', c=self.line_color)
        #        eMin = line([0, t[-1]], [-deltaE, -deltaE], lw=1,\
        #                            ls='--', c=self.line_color)        
        #        axes2.add_line(eMax)
        #        axes2.add_line(eMin)

        axes2.plot(t, eps, c='k')
        axes2.set_xlim(left=0, right=t[-1])
        #        axes2.set_ylim(top=0.4, bottom=-0.4)
        axes2.set_xlabel(r'$t [s]$', size=st.label_size)
        axes2.set_ylabel(r'$output error = x_{0} - y_{d} [m]$', size=st.label_size)

        self.calcMetrics(data, output)

        # check for sim succes
        if not data['results']['finished']:
            for key in list(output.keys()):
                output[key] = None

        # add settings and metrics to dictionary results
        results = {}
        results.update({'metrics': output})
        results.update({'modules': data['modules']})

        canvas = FigureCanvas(fig)

        self.writeOutputFiles(self.name, data['regime name'], fig, results)

        return {'name': '_'.join([data['regime name'], self.name]),
                'figure': canvas}

    def calcMetrics(self, data, output):
        '''
        calculate metrics for comaprism
        '''

        L1NormITAE = self.calcL1NormITAE(data)
        L1NormAbs = self.calcL1NormAbs(data)

        print('L1NormITAE: ', L1NormITAE)
        print('L1NormAbs: ', L1NormAbs)

        output.update({'L1NormITAE': L1NormITAE, 'L1NormAbs': L1NormAbs})


class eval_A2_epsilon(pm.PostProcessingModule):
    '''
    create diagrams like hauser did
    '''
    name = 'A2_epsilon'

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def run(self, data):
        print('processing ', data['regime name'])

        output = {}

        # calculate data-sets
        t = data['results']['simTime']
        yd = data['results']['trajectory_output.0']
        y = []
        for i in range(4):
            y.append(data['results']['model_output.' + str(i)])

        error = np.subtract(y[0], yd)
        output.update({'error': error})

        # plots
        fig = Figure()
        # fig.subplots_adjust(wspace=0.3, hspace=0.25)

        axes = fig.add_subplot(1, 1, 1)
        # axes.set_title(r'output error = yd - x0')
        axes.plot(t, error, c='k')
        axes.set_xlim(left=0, right=t[-1])
        axes.set_xlabel(r'$t [s]$')
        axes.set_ylabel(r'$output error = x_{0} - y_{d} [m]$')

        # calculate L1NormAbs
        L1NormAbs = self.calcL1NormAbs(data)
        output.update({'L1NormAbs': L1NormAbs})

        # check for sim succes
        if not data['results']['finished']:
            for key in list(output.keys()):
                output[key] = None

        # add settings and metrics to dictionary results
        results = {}
        results.update({'metrics': output})
        results.update({'modules': data['modules']})

        canvas = FigureCanvas(fig)

        self.writeOutputFiles(self.name, data['regime name'], fig, results)

        return {'name': '_'.join([data['regime name'], self.name]),
                'figure': canvas}


class eval_A2Hauser(pm.PostProcessingModule):
    '''
    create diagrams like hauser did
    '''

    name = 'A2_hauser'

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def run(self, data):
        results = {}
        output = {}
        print('processing ', data['regime name'])

        # calculate data-sets
        t = data['results']['simTime']
        yd = data['results']['trajectory_output.0']
        y = []
        for i in range(4):
            y.append(data['results']['model_output.' + str(i)])

        error = np.subtract(yd, y[0])

        # controlleroutput is a torque
        tau = data['results']['controller_output.0']
        # u in the neglected nonlinearity is theta_dd
        u = np.true_divide( \
            np.subtract( \
                np.subtract( \
                    tau, \
                    np.multiply( \
                        np.multiply( \
                            np.multiply( \
                                2 * st.M, \
                                y[0] \
                                ), \
                            y[1] \
                            ), \
                        y[3] \
                        ) \
                    ), \
                np.multiply( \
                    st.M * st.G, \
                    np.multiply( \
                        y[0], \
                        np.cos(y[2]) \
                        ) \
                    ) \
                ), \
            np.add(
                np.multiply( \
                    st.M, \
                    np.power(y[0], 2) \
                    ), \
                st.J + st.Jb \
                ) \
            )

        # Parameter from Controller -> modelling (estimate/meausre paramters)
        # and then neglect psi therm
        # if you are interested in the error through the negligence 
        if data['modules']['controller']['type'] == 'FController':
            psi = np.multiply(np.multiply(st.B, y[0]), np.power(y[3], 2))
        elif data['modules']['controller']['type'] == 'GController':
            psi = np.multiply(np.multiply(np.dot(2 * st.B, y[0]), y[3]), u)
        elif data['modules']['controller']['type'] == 'JController':
            psi = np.multiply(np.multiply(np.multiply(st.B, y[0]), np.power(y[3], 2)), \
                              np.multiply(st.B * st.G, np.subtract(y[2], np.sin(y[2]))))
        else:
            # psi is not defined in this case
            psi = np.dot(0, t)

        # plots
        fig = Figure()
        fig.subplots_adjust(wspace=0.3, hspace=0.25)

        axes1 = fig.add_subplot(2, 2, 1)
        # axes1.set_title(r'\textbf{output error = yd - x0}')
        axes1.plot(t, error, c='k')
        axes1.set_xlim(left=0, right=t[-1])
        axes1.set_xlabel(r'$t \,[s]$', size=st.label_size)
        axes1.set_ylabel(r'$e \,[m]$', size=st.label_size)

        axes2 = fig.add_subplot(2, 2, 2)
        # axes2.set_title(r'\textbf{Beam Angle}')
        axes2.plot(t, y[2], c='k')
        axes2.set_xlim(left=0, right=t[-1])
        axes2.set_xlabel(r'$t \, [s]$', size=st.label_size)
        axes2.set_ylabel(r'$\theta \, [rad]$', size=st.label_size)

        axes3 = fig.add_subplot(2, 2, 3)
        # axes3.set_title(r'\textbf{neglected nonlinearity}')
        axes3.plot(t, psi, c='k')
        axes3.set_xlim(left=0, right=t[-1])
        axes3.set_xlabel(r'$t [s]$', size=st.label_size)
        if data['modules']['controller']['type'] == 'FController':
            axes3.set_ylabel(r'$\psi_2 \, [\frac{m}{s^2}]$', size=st.label_size)
        if data['modules']['controller']['type'] == 'JController':
            axes3.set_ylabel(r'$\psi_2 \, [\frac{m}{s^2}]$', size=st.label_size)
        if data['modules']['controller']['type'] == 'GController':
            axes3.set_ylabel(r'$\psi_3 \, [\frac{m}{s^3}]$', size=st.label_size)

        axes4 = fig.add_subplot(2, 2, 4)
        # axes4.set_title(r'\textbf{Beam Torque}')
        axes4.plot(t, tau, c='k')
        axes4.set_xlim(left=0, right=t[-1])
        axes4.set_xlabel(r'$t \,[s]$', size=st.label_size)
        axes4.set_ylabel(r'$\tau \,[Nm]$', size=st.label_size)

        # calculate maximumError
        start = 30
        end = 40
        tStartIdx = next((idx for idx, x in enumerate(t) if x >= start), 0)
        tEndIdx = next((idx for idx, x in enumerate(t[start:]) if x >= end), len(t) - 1)

        maximumError = None
        if tStartIdx < tEndIdx:
            maximumError = max(error[tStartIdx:tEndIdx])

        print('maximum error between %d and %d seconds: %f' % \
              (start, end, maximumError))

        # collect results
        output.update({'maximumError': maximumError})

        # check for sim succes
        if not data['results']['finished']:
            for key in list(output.keys()):
                output[key] = None

        results.update({'metrics': output})
        results.update({'modules': data['modules']})

        # write results
        filePath = os.path.join(os.path.pardir, 'results', 'postprocessing', 'HauserDiagrams')
        if not os.path.isdir(filePath):
            os.makedirs(filePath)

        fileName = os.path.join(filePath, data['regime name'])
        with open(fileName + '.pof', 'w') as f:  # POF - Postprocessing Output File
            f.write(repr(results))

        canvas = FigureCanvas(fig)
        fig.savefig(fileName + '.svg')
        fig.savefig(fileName + '.png')
        fig.savefig(fileName + '.pdf')

        return {'name': '_'.join([data['regime name'], self.name]), \
                'figure': canvas}


class EvalA2HauserLikeHauser(pm.PostProcessingModule):
    """
    create diagrams like hauser did
    """

    name = 'A2_hauser'

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def process(self, files):
        print('process() of A2Hauser_LikeHauser called')
        results = {}
        output = {}
        l = []

        controllerDict = {'FController': [], \
                          'GController': [], \
                          'JController': []}
        for elem in controllerDict:
            controllerDict.update({elem: {'e': {}, 'theta': {}, 'psi': {}, 'tau': {}}})
            for var in controllerDict[elem]:
                for i in range(1, 4, 1):
                    controllerDict[elem][var].update({i: [[], []]})

        for res in files:

            # calculate data-sets
            t = res['results']['simTime']

            yd = res['results']['trajectory_output.0']
            y = []
            for i in range(4):
                y.append(res['results']['model_output.' + str(i)])

            error = np.subtract(yd, y[0])

            # controlleroutput is a torque
            tau = res['results']['controller_output.0']
            # u in the neglected nonlinearity is theta_dd
            u = np.true_divide( \
                np.subtract( \
                    np.subtract( \
                        tau, \
                        np.multiply( \
                            np.multiply( \
                                np.multiply( \
                                    2 * st.M, \
                                    y[0] \
                                    ), \
                                y[1] \
                                ), \
                            y[3] \
                            ) \
                        ), \
                    np.multiply( \
                        st.M * st.G, \
                        np.multiply( \
                            y[0], \
                            np.cos(y[2]) \
                            ) \
                        ) \
                    ), \
                np.add(
                    np.multiply( \
                        st.M, \
                        np.power(y[0], 2) \
                        ), \
                    st.J + st.Jb \
                    ) \
                )

            # Parameter from Controller -> modelling (estimate/meausre paramters)
            # and then neglect psi therm
            # if you are interested in the error through the negligence 
            if res['modules']['controller']['type'] == 'FController':
                psi = np.multiply(np.multiply(st.B, y[0]), np.power(y[3], 2))
            elif res['modules']['controller']['type'] == 'GController':
                psi = np.multiply(np.multiply(np.dot(2 * st.B, y[0]), y[3]), u)
            elif res['modules']['controller']['type'] == 'JController' or \
                            res['modules']['controller']['type'] == 'LSSController':
                psi = np.multiply(np.multiply(np.multiply(st.B, y[0]), np.power(y[3], 2)), \
                                  np.multiply(st.B * st.G, np.subtract(y[2], np.sin(y[2]))))
            else:
                # psi is not defined in this case
                psi = np.dot(0, t)

            # Dict füllen
            level1 = res['modules']['controller']['type']
            level3 = res['modules']['trajectory']['Amplitude']

            # Zeit anhängen
            #            controllerDict[level1].update({'t':t})
            controllerDict[level1]['e'][level3][0] = t
            controllerDict[level1]['theta'][level3][0] = t
            controllerDict[level1]['psi'][level3][0] = t
            controllerDict[level1]['tau'][level3][0] = t
            #            # Werte anhängen           
            controllerDict[level1]['e'][level3][1] = error
            controllerDict[level1]['theta'][level3][1] = y[2]
            controllerDict[level1]['psi'][level3][1] = psi
            controllerDict[level1]['tau'][level3][1] = tau


            # Plots erzeugen
        #        contr = ['FController', 'GController', 'JController', 'LSSController']
        contr = ['FController', 'GController', 'JController']
        for c in contr:
            print('controller:', c)

            fig = Figure()
            #            fig.subplots_adjust(wspace=0.3, hspace=0.25) 
            fig.subplots_adjust(wspace=0.6, hspace=0.6)
            fig.suptitle(r'\textbf{' + c + '}', size=st.title_size)

            for i in range(1, 4, 1):
                axes1 = fig.add_subplot(2, 2, 1)
                axes1.set_title(r'output error = yd - x0', size=st.label_size)
                axes1.plot(controllerDict[c]['e'][i][0], controllerDict[c]['e'][i][1])
                axes1.set_xlim(left=0, right=controllerDict[c]['e'][1][0][-1])
                axes1.set_xlabel(r'$t \,[s]$', size=st.label_size)
                axes1.set_ylabel(r'$e \,[m]$', size=st.label_size)
                axes1.grid(color='#ababab', linestyle='--')

                axes2 = fig.add_subplot(2, 2, 2)
                axes2.set_title(r'beam angle', size=st.label_size)
                axes2.plot(controllerDict[c]['theta'][i][0], controllerDict[c]['theta'][i][1])
                axes2.set_xlim(left=0, right=controllerDict[c]['theta'][1][0][-1])
                axes2.set_xlabel(r'$t \, [s]$', size=st.label_size)
                axes2.set_ylabel(r'$\theta \, [rad]$', size=st.label_size)
                axes2.grid(color='#ababab', linestyle='--')

                axes3 = fig.add_subplot(2, 2, 3)
                axes3.set_title(r'neglected nonlinearity', size=st.label_size)
                axes3.plot(controllerDict[c]['psi'][i][0], controllerDict[c]['psi'][i][1])
                axes3.set_xlim(left=0, right=controllerDict[c]['psi'][1][0][-1])
                axes3.set_xlabel(r'$t [s]$', size=st.label_size)
                if res['modules']['controller']['type'] == 'FController':
                    axes3.set_ylabel(r'$\psi_2 \, [\frac{m}{s^2}]$', size=st.label_size)
                if res['modules']['controller']['type'] == 'JController':
                    axes3.set_ylabel(r'$\psi_2 \, [\frac{m}{s^2}]$', size=st.label_size)
                    #                axes3.set_ylim(top = 0.6, bottom = -0.6)
                if res['modules']['controller']['type'] == 'GController':
                    axes3.set_ylabel(r'$\psi_3 \, [\frac{m}{s^3}]$', size=st.label_size)
                axes3.grid(color='#ababab', linestyle='--')

                axes4 = fig.add_subplot(2, 2, 4)
                axes4.set_title(r'beam torque', size=st.label_size)
                axes4.plot(controllerDict[c]['tau'][i][0], controllerDict[c]['tau'][i][1])
                axes4.set_xlim(left=0, right=controllerDict[c]['tau'][1][0][-1])
                axes4.set_xlabel(r'$t \,[s]$', size=st.label_size)
                axes4.set_ylabel(r'$\tau \,[Nm]$', size=st.label_size)
                axes4.grid(color='#ababab', linestyle='--')

                # calculate maximumError
                start = 30
                end = 40
                tStartIdx = next((idx for idx, x in enumerate(controllerDict[c]['e'][i][0]) if x >= start), 0)
                tEndIdx = next((idx for idx, x in enumerate(controllerDict[c]['e'][i][0][start:]) if x >= end), \
                               len(controllerDict[c]['e'][i][0]) - 1)

                maximumError = None
                if tStartIdx < tEndIdx:
                    maximumError = max(controllerDict[c]['e'][i][1][tStartIdx:tEndIdx])

                print('maximum error between %d and %d seconds: %f' % \
                      (start, end, maximumError))

            # check for sim succes
            if not res['results']['finished']:
                for key in list(output.keys()):
                    output[key] = None

            results.update({'metrics': output})
            results.update({'modules': res['modules']})

            # write results
            filePath = os.path.join(os.path.pardir, 'results', 'postprocessing', 'HauserDiagramsLikeHauser')
            if not os.path.isdir(filePath):
                os.makedirs(filePath)

            fileName = os.path.join(filePath, c)  # res['regime name'])
            with open(fileName + '.pof', 'w') as f:  # POF - Postprocessing Output File
                f.write(repr(results))

            canvas = FigureCanvas(fig)
            fig.savefig(fileName + '.svg')
            fig.savefig(fileName + '.png')
            fig.savefig(fileName + '.pdf')

            #            l.append({'name':'_'.join([res['regime name'], self.name]),\
            #                        'figure': canvas})
            l.append({'name': '_'.join([c, self.name]), \
                      'figure': canvas})

        return l


class EvalA3(pm.PostProcessingModule):
    """
    create diagrams for evaluation step A3
    """

    name = 'A3'

    line_color = '#aaaaaa'
    line_style = '-'
    font_size = 20
    epsPercent = 2.5
    spacing = 0.01
    counter = 0

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def run(self, data):
        print('processing ', data['regime name'])

        # dict for calculated values
        output = {}

        # reset counter
        self.counter = 0

        # calculate data-sets
        t = data['results']['simTime']
        y = data['results']['model_output.0']
        traj = data['results']['trajectory_output.0']
        yd = data['results']['trajectory_output.0'][-1]

        self.posLabel = np.arange(np.min(y) + 0.1 * yd, yd, (yd - np.min(y)) / 4)

        # create plot
        fig = Figure()
        axes = fig.add_subplot(111)
        axes.set_title(r'\textbf{Vergleich Signalverlaeufe (Systemantwort und Trajektorie)}')
        axes.plot(t, traj, c='b', ls='-', label='w(t)')
        # create t_desired line
        # search time value for t_desired
        t_desired = t[traj.index(yd)]
        self.createTimeLine(axes, t, traj, t_desired, r'$T_{des}$')
        # add real t_desired to dict output
        output.update({'t_desired': data['modules']['trajectory']['delta t']})
        # plot y(t)
        axes.plot(t, y, c='k', ls='-', label='y(t)')
        # axes scaling
        axes.set_xlim(left=0, right=t[-1])
        y_min = np.min(y)
        y_max = np.max(y)
        if (np.max(y) == 0):
            axes.set_ylim(np.min(traj), np.max(traj) + np.max(traj) * 0.1)
        else:
            axes.set_ylim(y_min, y_max + y_max * 0.1)
        axes.set_xlabel(r'\textit{Zeit [s]}')
        axes.set_ylabel(r'\textit{Ballposition r(t) [m]}')
        axes.legend(loc=4)

        # create desired line
        desiredLine = Line2D([0, t[-1]], [yd, yd], lw=1, ls=self.line_style, c='k')
        axes.add_line(desiredLine)

        # calc damping-time (Beruhigungszeit)
        eps = self.epsPercent * yd / 100
        enterIdx = -1
        for idx, val in enumerate(y):
            if enterIdx == -1:
                if abs(val - yd) < eps:
                    enterIdx = idx
            else:
                if abs(val - yd) >= eps:
                    enterIdx = -1
        if enterIdx == -1:
            # print 'DampingLine is not defined'
            output.update({'td': None})
        else:
            td = t[enterIdx]
            # create and add line
            self.createTimeLine(axes, t, y, td, r'$T_{\epsilon}$')
            output.update({'td': td})

        # create epsilon tube
        upperBoundLine = Line2D([0, t[-1]], [yd + eps, yd + eps], ls='--', c=self.line_color)
        axes.add_line(upperBoundLine)
        lowerBoundLine = Line2D([0, t[-1]], [yd - eps, yd - eps], ls='--', c=self.line_color)
        axes.add_line(lowerBoundLine)

        # calc control deviation
        control_deviation = y[-1] - yd
        output.update({'control_deviation': control_deviation})

        self.calcMetrics(data, output)

        # check for sim sucess
        if not data['results']['finished']:
            for key in output.keys():
                output[key] = None

        # add settings and metrics to dictionary results
        results = {}
        results.update({'metrics': output})
        results.update({'modules': data['modules']})

        canvas = FigureCanvas(fig)

        self.writeOutputFiles(self.name, data['regime name'], fig, results)

        return {'name': '_'.join([data['regime name'], self.name]), \
                'figure': canvas}

    def createTimeLine(self, axes, t, y, time_value, label):
        if time_value != t[-1]:
            # create timeLine
            timeLine = Line2D([time_value, time_value], \
                              [axes.get_ylim()[0], y[t.index(time_value)]], \
                              ls=self.line_style, \
                              c=self.line_color)
            axes.add_line(timeLine)
            # create label
            axes.text(time_value + self.spacing, self.posLabel[self.counter], label, size=self.font_size)
            self.counter = self.counter + 1

    def calcMetrics(self, data, output):
        '''
        calculate metrics for comaprism
        '''

        # calculate time difference
        if output['td'] == None:
            t_diff = None
        else:
            t_diff = output['td'] - data['modules']['trajectory']['delta t']

        # extract delta_t
        delta_t = data['modules']['trajectory']['delta t']

        L1NormITAE = self.calcL1NormITAE(data)
        L1NormAbs = self.calcL1NormAbs(data)

        print('L1NormITAE: ', L1NormITAE)
        print('L1NormAbs: ', L1NormAbs)

        output.update({'L1NormITAE': L1NormITAE, \
                       'L1NormAbs': L1NormAbs, \
                       'delta_t': delta_t, \
                       't_diff': t_diff, \
                       })


class EvalB(pm.PostProcessingModule):
    """
    create diagrams for evaluation step B
    """

    name = 'B'
    # padding = .2
    padding = .5
    offset = 0

    line_color = '#aaaaaa'
    line_style = '-'
    font_size = 20
    epsPercent = 2.5

    # spacing = 0.01
    # counter = 0

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def process(self, dataList):
        print('process() of B called')
        output = {}

        # ideal regime name
        regName = next((result['regime name'] \
                        for result in dataList if 'ideal' in result['regime name']), None)

        # extract the needed curves
        t = self.extractValues(dataList, 'ideal', 'simTime')
        y_ideal = self.extractValues(dataList, 'ideal', 'model_output.0')
        y_desired = self.extractValues(dataList, 'ideal', 'trajectory_output.0')
        yd = y_desired[-1]
        y_pTolMin = self.extractValues(dataList, 'paramTolMin', 'model_output.0')
        y_pTolMax = self.extractValues(dataList, 'paramTolMax', 'model_output.0')

        par = next((param for param in st.paramVariationListB if param in regName), None)
        print('assuming that', par, 'has been varied.')

        # sort files by variied parameter
        modDataList = sorted(dataList, key=lambda k: k['modules']['model'][par], reverse=False)

        # find minimal stable iteration
        resAbsMin = next((res \
                          for res in modDataList if res['results']['finished']), None)
        y_pAbsMin = resAbsMin['results']['model_output.0']

        # find maximum stable iteration
        resAbsMax = next((res \
                          for res in reversed(modDataList) if res['results']['finished']), None)
        y_pAbsMax = resAbsMax['results']['model_output.0']
        # print len(y_pAbsMin), len(y_pAbsMax), len(y_ideal)

        print('stablity limits are:', \
              resAbsMin['modules']['model'][par], '/', resAbsMax['modules']['model'][par])

        output.update({'parameter': par, \
                       'minLimit': resAbsMin['modules']['model'][par], \
                       'maxLimit': resAbsMax['modules']['model'][par], \
                       })

        # create plot
        fig = Figure()
        axes = fig.add_subplot(111)
        axes.set_title('Vergleich Signalverläufe')

        # create epsilon tub
        # eps = self.epsPercent*yd/100
        eps = st.R

        upperBoundLine = Line2D([0, t[-1]], [yd + eps, yd + eps], ls='--', c=self.line_color)
        axes.add_line(upperBoundLine)
        lowerBoundLine = Line2D([0, t[-1]], [yd - eps, yd - eps], ls='--', c=self.line_color)
        axes.add_line(lowerBoundLine)

        # create signals
        axes.add_line(lowerBoundLine)
        axes.plot(t, y_desired, c='b', ls='-', label='w(t)')
        axes.plot(t, y_ideal, c='limegreen', ls='-', label='exakter Wert')
        axes.plot(t, y_pTolMin, c='indianred', ls='-', label='untere Toleranzgrenze')
        axes.plot(t, y_pTolMax, c='darkorange', ls='-', label='obere Toleranzgrenze')
        # axes.plot(t, y_pAbsMin, c='orange', ls='-', label='untere Stabilitaetsgrenze')
        # axes.plot(t, y_pAbsMax, c='r', ls='-', label='obere Stabilitaetsgrenze')

        # customize
        axes.set_xlim(left=0, right=t[-1])
        #        axes.set_ylim(bottom=(self.offset+yd*(1-self.padding/2)),\
        #                top=(self.offset+yd*(1+self.padding)))

        axes.legend(loc=0, fontsize='small')
        axes.set_xlabel(r'$t \, \lbrack s \rbrack$')
        axes.set_ylabel(r'$r \, \lbrack m \rbrack$')

        canvas = FigureCanvas(fig)
        extendedName = os.path.join(self.name, par)

        # create output files because run is not called
        for data in dataList:
            results = {}
            results.update({'metrics': {}})
            self.calcMetrics(data, results['metrics'])

            # add settings and metrics to dictionary results
            results.update({'modules': data['modules']})

            self.writeOutputFiles(extendedName, data['regime name'], None, results)

        results = {'metrics': output}

        # create new dir for parameter
        self.writeOutputFiles(extendedName, 'paramLimits_' + regName[:-len('ideal')], \
                              fig, results)

        return [{'name': '_'.join([regName[:-len('ideal')], 'paramLimits', self.name]), \
                 'figure': canvas}]

    def calcMetrics(self, data, output):
        '''
        calculate metrics for comaprism
        '''
        # calc L1NormAbs
        L1NormAbs = self.calcL1NormAbs(data)

        # calc L1NormITAE
        L1NormITAE = self.calcL1NormITAE(data)
        output.update({ \
            'L1NormITAE': L1NormITAE, \
            'L1NormAbs': L1NormAbs, \
            })


class EvalC2(pm.PostProcessingModule):
    """
    create diagrams for evaluation step C - limitations
    """

    # padding = .2
    padding = .5
    offset = 0

    line_color = '#aaaaaa'
    line_style = '-'
    font_size = 20

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def process(self, dataList):
        print('process() of C2 called')

        output = []
        for cName in st.smoothPoles.keys():
            # check whether controller was in result files
            t = [None, None]
            t[0] = self.extractValues(dataList, [cName, '_unlimited'], 'simTime')
            t[1] = self.extractValues(dataList, [cName, '_limited'], 'simTime')
            if not (t[0] and t[1]):
                continue

            timeIdx = 0
            if len(t[1]) > len(t[0]):
                # limited controller lastet longer
                timeIdx = 1

            print('found entry of ', cName)

            # get curves
            r = [None, None]
            y = [None, None]
            ydes = [None, None]
            r[0] = self.extractValues(dataList, [cName, '_unlimited'], 'controller_output.0')
            r[1] = self.extractValues(dataList, [cName, '_limited'], 'controller_output.0')
            l = self.extractValues(dataList, [cName, '_limited'], 'limiter_output.0')
            ydes[0] = self.extractValues(dataList, [cName, '_unlimited'], 'trajectory_output.0')
            ydes[1] = self.extractValues(dataList, [cName, '_limited'], 'trajectory_output.0')
            ydEnd = ydes[timeIdx][-1]
            y[0] = self.extractValues(dataList, [cName, '_unlimited'], 'model_output.0')
            y[1] = self.extractValues(dataList, [cName, '_limited'], 'model_output.0')

            # get settings
            limits = self.extractSetting(dataList, [cName, '_limited'], 'limiter', 'limits')
            eps = st.R

            # create plot
            fig = Figure()
            fig.subplots_adjust(hspace=0.4)
            axes = []

            # fig1 controller output
            axes.append(fig.add_subplot(211))
            axes[0].set_title("Reglerausgänge im Vergleich")

            # create limitation tube
            upperBoundLine = Line2D([0, t[timeIdx][-1]], [limits[0]] * 2, ls='--', c=self.line_color)
            axes[0].add_line(upperBoundLine)
            lowerBoundLine = Line2D([0, t[timeIdx][-1]], [limits[1]] * 2, ls='--', c=self.line_color)
            axes[0].add_line(lowerBoundLine)

            axes[0].plot(t[0], r[0], c='limegreen', ls='-', label='r(t) unlimitriert')
            axes[0].plot(t[1], r[1], c='indianred', ls='-', label='r(t) limitiert')

            # customize
            axes[0].set_xlim(left=0, right=t[timeIdx][-1])
            #        axes.set_ylim(bottom=(self.offset+yd*(1-self.padding/2)),\
            #                top=(self.offset+yd*(1+self.padding)))

            axes[0].legend(loc=0, fontsize='small')
            axes[0].set_xlabel(r'$t \, \lbrack s \rbrack$')
            axes[0].set_ylabel(r'$\tau \, \lbrack Nm \rbrack$')

            # fig2 model output
            axes.append(fig.add_subplot(212))
            axes[1].set_title("Ausgangsverläufe im Vergleich")

            # create epsilon tube
            upperBoundLine = Line2D([0, t[timeIdx][-1]], [ydEnd + eps, ydEnd + eps], ls='--', c=self.line_color)
            axes[1].add_line(upperBoundLine)
            lowerBoundLine = Line2D([0, t[timeIdx][-1]], [ydEnd - eps, ydEnd - eps], ls='--', c=self.line_color)
            axes[1].add_line(lowerBoundLine)

            axes[1].plot(t[timeIdx], ydes[timeIdx], c='b', ls='-', label='w(t)')
            axes[1].plot(t[0], y[0], c='limegreen', ls='-', label='y(t) unlimitriert')
            axes[1].plot(t[1], y[1], c='indianred', ls='-', label='y(t) limitiert')

            # customize
            axes[1].set_xlim(left=0, right=t[timeIdx][-1])
            #        axes.set_ylim(bottom=(self.offset+yd*(1-self.padding/2)),\
            #                top=(self.offset+yd*(1+self.padding)))

            axes[1].legend(loc=0, fontsize='small')
            axes[1].set_xlabel(r'$t \, \lbrack s \rbrack$')
            axes[1].set_ylabel(r'$r \, \lbrack m \rbrack$')

            canvas = FigureCanvas(fig)

            # create output files because run is not called
            data_sets = [dataSet for dataSet in dataList if cName in dataSet['regime name']]
            for data in data_sets:
                results = {}
                results.update({'metrics': {}})
                self.calcMetrics(data, results['metrics'])

                # add settings and metrics to dictionary results
                results.update({'modules': data['modules']})
                if '_unlimited' in data['regime name']:
                    appendix = '_unlimited'
                else:
                    appendix = '_limited'
                self.writeOutputFiles(self.name, data['regime name'], fig, results)

            output.append({'name': '_'.join([cName, self.name]), \
                           'figure': canvas})
        return output

    def calcMetrics(self, data, output):
        '''
        calculate metrics for comaprism
        '''
        # calc L1NormAbs
        L1NormAbs = self.calcL1NormAbs(data)

        # calc L1NormITAE
        L1NormITAE = self.calcL1NormITAE(data)
        output.update({'L1NormITAE': L1NormITAE,
                       'L1NormAbs': L1NormAbs,
                       })


class EvalCObserver(pm.PostProcessingModule):
    """
    create diagrams like hauser did
    """
    line_color = '#aaaaaa'
    line_style = '-'

    name = 'C_observer'

    def __init__(self):
        pm.PostProcessingModule.__init__(self)
        return

    def run(self, data):
        print('processing ', data['regime name'])

        output = {}

        # vectorise skalar functions
        vSubt = np.vectorize(self.subt)

        # calculate data-sets
        t = data['results']['simTime']
        xm = []
        xo = []
        e = []
        for i in range(4):
            xm.append(data['results']['model_output.' + str(i)])
            xo.append(data['results']['observer_output.' + str(i)])
            e.append(vSubt(xo[i], xm[i]))

        # plots
        fig = Figure()
        fig.subplots_adjust(wspace=0.3, hspace=0.25)

        axes = []
        for i in range(4):
            axes.append(fig.add_subplot(5, 1, i + 1))
            axes[i].plot(t, xo[i], c='b', ls='-', label='xo[' + str(i) + ']')
            axes[i].plot(t, xm[i], c='k', ls='-', label='xm[' + str(i) + ']')
            axes[i].set_xlim(left=0, right=t[-1])

        axes.append(fig.add_subplot(5, 1, 5))
        leg = []
        for i in range(4):
            axes[4].plot(t, e[i], ls='-', label='e[' + str(i) + ']')
            leg.append('e[' + str(i) + ']')

        axes[4].legend(leg, loc=0, fontsize='small')

        # calculate results
        # L1 
        dt = 1.0 / data['modules']['solver']['measure rate']
        errorIntegrals = [0, 0, 0, 0]

        # check for sim success
        if not data['results']['finished']:
            for key in output.keys():
                output[key] = None

        for i in range(len(errorIntegrals)):
            for k, val in enumerate(xo[i]):
                # vgl. Betragskriterium L^1
                errorIntegrals[i] += 1
                errorIntegrals[i] += abs(val - xm[i][k]) * dt

            print('errorIntegral_x[' + str(i) + ']:', errorIntegrals[i])
            output.update({'error_L1Norm_x[' + str(i) + ']': errorIntegrals[i]})

        # add settings and metrics to dictionary results
        results = {}
        results.update({'metrics': output})
        results.update({'modules': data['modules']})

        canvas = FigureCanvas(fig)

        self.writeOutputFiles(self.name, data["regime name"], fig, results)

        return dict(name="_".join({data["regime name"], self.name}), figure=canvas)

# pm.register_processing_module(pm.PostProcessingModule, EvalA1)



#!/usr/bin/python2
# -*- coding: utf-8 -*-

import numpy as np
import os

controllerList = ['FController', 'GController', 'JController',\
                'LSSController', 'PIFeedbackController']
parameterList = ['Jb', 'M']
lines = ''

print('Regimefile Generator')
print('Choose one of the following parameters: (0-1)')
print(parameterList)
number = -1
while not(number >= 0 and number <= 4):
    print('Number: ')
    number = eval(input())

parameter = parameterList[number]


# load head file
filePath = os.path.join(os.curdir, 'B_head.sray')
with open(filePath, 'r') as f:
    head = f.read()

lines += '\n'

# how many poles?
if controller == 'PIFeedbackController':
    multiplicator = 5
else:
    multiplicator = 4


if controller == 'FController':
    pole = -3.6
elif controller == 'GController':
    pole = -3
elif controller == 'JController':
    pole == -2
elif controller == 'LSSController':
    pole = -3.3
elif controller == 'PIFeedbackController':
    pole = -1.5

Jb = np.arange(0.5e-6, 10.5e-6, 1e-6)
    
for jb in Jb:
        lines += '- name: B1_' + controller + '_poles(' + str(pole) \
                    + ')_Jb(' + str(jb) + ')\n'
        lines += '  clear previous: !!python/bool False' + '\n'
        lines += '\n'
        lines += '  model:' + '\n'
        lines += '   type: BallBeamModel' + '\n'
        lines += '   Jb: ' + str(jb) + '\n'
        lines += '\n'
        lines += '  controller: ' + '\n'
        lines += '   type: '  + controller + '\n'
        lines += '   poles: ' + str([pole]*multiplicator) + '\n'
        if (controller == 'PIFeedbackController' or controller == 'LSSController'):
            lines += '   r0: 3'
        lines += '\n'

fileName = 'B1_' + controller + '_poles' + str(pole) \
                    + '_Jb(' + str(Jb[0]) + ',' + str(Jb[-1]) + ').sreg'
filePath = os.path.join(os.curdir, fileName)
with open(filePath, 'w') as f:
    f.write(head)    
    f.write(lines)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read().replace(".. :changelog:", "")

with open("requirements.txt") as requirements_file:
    requirements = requirements_file.read()

test_requirements = [
]

extra_requirements = {
    "3d": ["vtk>=6.3.0"],
    "cpp": ["pybind11>=2.7.0"],
    "docs": ["Sphinx>=1.4.9",
             "sphinx-rtd-theme>=0.1.9",
             "sphinxcontrib-tikz>=0.4.7",
             "docutils",
             ],
    "test": [],
}

setup(
    name="pymoskito",
    version="0.4.0",
    description="Python based modular simulation & postprocessing kickass "
                "toolbox",
    long_description=readme + "\n\n" + history,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="pymoskito control simulation feedback feedforward",
    long_description_content_type="text/x-rst",
    url="https://github.com/cklb/pymoskito",
    author="Stefan Ecklebe",
    author_email="stefan.ecklebe@tu-dresden.de",
    license="GPLv3",
    packages=["pymoskito"],
    package_dir={"pymoskito": "pymoskito"},
    install_requires=requirements,
    include_package_data=True,
    test_suite="pymoskito.tests",
    extras_require=extra_requirements,
    zip_safe=False
)



# -*- coding: utf-8 -*-


import logging
import sys

import numpy as np
from PyQt5.QtCore import QObject, pyqtSignal, pyqtSlot

from .simulation_modules import SimulationException


class SimulationStateChange(object):
    """
    Object that is emitted when Simulator changes its state.
    
    Keyword Args:
        type: Keyword describing the state change, can be one of the following
        
            * `init` Initialisation
            * `start` : Start of Simulation
            * `time` : Accomplishment of new progress step
            * `finish` : Finish of Simulation
            * `abort` : Abortion of Simulation
            
        data: Data that is emitted on state change.
        info: Further information.
        
    """

    def __init__(self, **kwargs):

        assert "type" in kwargs.keys()
        for key, val in kwargs.items():
            setattr(self, key, val)


class SimulationSettings(object):
    def __init__(self, start_time, end_time, step_size, measure_rate):
        self.start_time = start_time
        self.end_time = end_time
        self.step_size = step_size
        self.measure_rate = measure_rate

    def to_dict(self):
        return {
            "start time": self.start_time,
            "end time": self.end_time,
            "step size": self.step_size,
            "measure rate": self.measure_rate,
        }


class Simulator(QObject):
    """ 
    This Class executes the time-step integration.

    It forms the Core of the physical simulation and interacts with the GUI
    via the :py:class:''SimulationInterface`

    Calculated values will be stored every 1 / measure rate seconds.
    """

    work_done = pyqtSignal()
    state_changed = pyqtSignal(SimulationStateChange)

    # list of modules that have to appear in every run
    static_module_list = [
        "Model",
        "Solver"
    ]

    # list of modules that might not always appear but have to be calculated
    # in a special order
    _dynamic_module_list = [
        "Disturbance",
        "Sensor",
        "ObserverMixer",
        "Observer",
        "Trajectory",
        "Feedforward",
        "Controller",
        "ModelMixer",
        "Limiter",
    ]

    module_list = static_module_list + _dynamic_module_list

    def __init__(self, settings, modules):
        QObject.__init__(self, None)
        self._run = False
        self._logger = logging.getLogger(self.__class__.__name__)

        assert isinstance(settings, SimulationSettings)
        self._settings = settings
        assert isinstance(modules, dict)
        self._simulation_modules = modules

        self._init_states()
        self._init_settings()
        self.updated_time = 0
        self._storage = dict()

    def _init_states(self):
        self._input_vector = {}
        self._counter = {}
        self._current_outputs = {}
        self._current_outputs.update(time=self._settings.start_time)

        for mod_name, obj in self._simulation_modules.items():
            self._counter.update({mod_name: obj.tick_divider})
            self._current_outputs.update({mod_name: []})
            self._current_outputs[mod_name] = []

        # init model output with current state
        self._simulation_modules["Solver"].next_output = np.array(
            self._simulation_modules["Model"].initial_state)

    def _init_settings(self):
        """ Initialize module settings that depend on other modules.
        """
        # calculate the correct step width for every block
        for mod_name, obj in self._simulation_modules.items():
            obj.step_width = obj.tick_divider * self._settings.step_size

        return

    def _calc_module(self, module_name):
        """ Calculates the output of a simulation module
        """
        if module_name in self._simulation_modules.keys():
            if self._counter[module_name] == \
                    self._simulation_modules[module_name].tick_divider:
                self._current_outputs[module_name] = np.atleast_1d(
                    self._simulation_modules[module_name].calc_output(
                        self._input_vector))
                self._counter[module_name] = 1
            else:
                self._counter[module_name] += 1

            # update input vector
            self._input_vector.update(
                {module_name: self._current_outputs[module_name]})

    def _calc_step(self):
        """
        Calculate one step in simulation.
        
        Warn:
            Due to the observers need for the last system input, the values of
            the last step are kept in the input vector until they are 
            overridden. Be careful about which value is needed at which place or
            otherwise you end up using a value from the last step.
        """
        # update time and current state
        self._current_outputs["time"] = self._simulation_modules["Solver"].t
        self._input_vector.update(
            time=self._current_outputs["time"],
            system_state=np.atleast_1d(
                self._simulation_modules["Solver"].next_output)
        )

        # apply new output
        self._current_outputs["Model"] = np.atleast_1d(
            self._simulation_modules["Model"].calc_output(
                self._input_vector["system_state"]))
        self._input_vector.update(system_output=self._current_outputs["Model"])

        # compute all dynamic modules
        for mod in self._dynamic_module_list:
            self._calc_module(mod)

        # integrate model
        self._choose_system_input(self._input_vector)
        self._calc_module("Solver")

        return

    def _choose_system_input(self, input_vector):
        """ This is mainly done for convenience.
        """
        if "Limiter" in input_vector:
            _input = input_vector["Limiter"]
        elif "ModelMixer" in input_vector:
            _input = input_vector["ModelMixer"]
        elif "Controller" in input_vector:
            if "Feedforward" in input_vector:
                raise SimulationException(
                    "Controller and Feedforward present but no"
                    "ModelMixer. Ambiguous Situation")
            _input = input_vector["Controller"]
        elif "Feedforward" in input_vector:
            _input = input_vector["Feedforward"]
        else:
            raise SimulationException("No system input given.")

        self._input_vector["system_input"] = _input

    def _store_values(self):
        """
        store all values of finished integration step
        """
        for key, val in self._current_outputs.items():
            if key in self._storage:
                self._storage[key].append(np.array(val))
            else:
                self._storage.update({key: [np.array(val)]})

        return

    def _check_time(self):
        """
        send update notification every second
        """
        t = self._current_outputs["time"]
        if t - self.updated_time > 1:
            self.state_changed.emit(SimulationStateChange(type="time", t=t))
            # self.timeChanged.emit(t)
            self.updated_time = t

    @pyqtSlot()
    def run(self):
        """
        Start the simulation.
        """
        self._run = True
        self.state_changed.emit(SimulationStateChange(type="start"))

        first_run = True
        rate = 1 / self._settings.measure_rate
        solver = self._simulation_modules["Solver"]

        while self._current_outputs["time"] < self._settings.end_time:
            t = solver.t
            dt = 0
            while dt < rate:
                if not self._run:
                    self._abort("Simulation aborted by user")
                    break

                try:
                    self._calc_step()

                except Exception:
                    # catch all to avoid loosing data
                    self._abort(sys.exc_info())
                    return

                dt = solver.t - t
                if dt < rate and first_run:
                    self._store_values()
                first_run = False

            self._store_values()
            self._check_time()

        self._finish()

    def _abort(self, info):
        """ Overwrite end time with reached time.
        """
        self._settings.end_time = self._current_outputs["time"]
        self._storage.update(finished=False)
        end_state = "abort"
        self.state_changed.emit(SimulationStateChange(type=end_state,
                                                      data=self.output,
                                                      info=info))
        self.work_done.emit()

    def _finish(self):
        self._storage.update(finished=True)
        end_state = "finish"
        self.state_changed.emit(SimulationStateChange(type=end_state,
                                                      data=self.output,
                                                      info="Success"))
        self.work_done.emit()

    @pyqtSlot(name="stop")
    def stop(self):
        """ Stop the simulation. """
        self._run = False

    @property
    def output(self):
        # convert storage entries
        out = dict(modules={}, simulation={}, results={})

        for mod, results in self._storage.items():
            # grab module settings
            if mod in self._simulation_modules:
                out["modules"].update(
                    {mod: self._simulation_modules[mod].settings})

            # grab module data
            if not isinstance(results, list):
                # flag or string -> nothing to convert
                entry = results
            elif isinstance(results[0], np.ndarray):
                # convert list of 1d-arrays into 2d-array
                entry = np.array(results)
            else:
                # convert list of scalars into 1d-array
                entry = np.array(results)
            out["results"].update({mod: entry})

        # grab simulator settings
        out.update({"simulation": self._settings.to_dict()})
        return out

    @property
    def settings(self):
        return self._settings.to_dict()



"""
This file contains some functions that are quite helpful when designing feedback
laws.
This collection is not complete and does not aim to be so. 
For a more sophisticated collection have a look at the `symbtools`
(https://github.com/TUD-RST/symbtools) or `control` package which are not used
in this package to keep a small footprint.
"""

import warnings
import sympy as sp
import numpy as np
from numpy.linalg import inv as mat_inv

__all__ = ["char_coefficients", "place_siso", "calc_prefilter",
           "controllability_matrix", "observability_matrix",
           "lie_derivatives"
           ]


def lie_derivatives(h, f, x, order=1):
    """
    Calculates the Lie-Derivative from a scalar field :math:`h(x)` along a 
    vector field :math:`f(x)`.
    
    Args:
        h (sympy.matrix): scalar field
        f (sympy.matrix): vector field
        x (sympy.matrix): symbolic representation of the states
        order (int): order

    Return:
        list of sympy.matrix: lie derivatives in ascending order
    """
    derivatives = [h]
    for i in range(order):
        derivatives.append(derivatives[-1].jacobian(x) * f)

    return derivatives


def char_coefficients(poles):
    """
    Calculate the coefficients of a characteristic polynomial.
    
    Args:
        poles (list or :obj:`numpy.ndarray`): pol configuration

    Return:
        :obj:`numpy.ndarray`: coefficients
    """

    poles = np.array(poles)  # convert to numpy array
    poles = np.ravel(poles)  # transform to 1d array

    s = sp.symbols("s")
    poly = 1
    for s_i in poles:
        poly = (s - s_i) * poly
    poly = sp.expand(poly)

    # calculate the coefficient of characteristic polynomial
    n = len(poles)
    p = []
    for i in range(n):
        p.append(poly.subs([(s, 0)]))
        poly = poly - p[i]
        poly = poly / s
        poly = sp.expand(poly)

    # convert numbers and complex objects from multiplication to a complex
    # number
    p = [complex(x) for x in p]

    # if imaginary part is greater than the boundary, set imaginary part to zero
    boundary = 1e-12
    for idx, val in enumerate(p):
        if abs(val.imag) > boundary:
            msg = "Imaginary Part of the coefficient p[{}] "
            "is unequal zero ({})) for a given tolerance of {}".format(
                str(idx), str(boundary), str(val.imag))
            warnings.warn(msg)
        p[idx] = val.real

    return np.array(p, dtype=float)  # [a_0, a_1, ... , a_n-1]


def controllability_matrix(a_mat, b_mat):
    """
    Calculate controllability matrix and check controllability of the system.

    .. math::
        \\boldsymbol{Q_{c}} = \\begin{pmatrix}
        \\boldsymbol{B} & \\boldsymbol{A}\\boldsymbol{B} & \\boldsymbol{A}^{2}
        \\boldsymbol{B} &
        \\cdots & \\boldsymbol{A}^{n-1}\\boldsymbol{B}\\\\
        \\end{pmatrix}

    Args:
        a_mat (:obj:`numpy.ndarray`): system matrix
        b_mat (:obj:`numpy.ndarray`): manipulating matrix

    Return:
        :obj:`numpy.ndarray`: controllability matrix :math:`\\boldsymbol{Q_{c}}`
    """
    a_mat = np.atleast_2d(a_mat)
    b_mat = np.atleast_2d(b_mat)

    # check dimension of matrix A and B
    if a_mat.shape[0] != a_mat.shape[1]:
        raise ValueError("A is not square")
    if a_mat.shape[0] != b_mat.shape[0]:
        raise ValueError("Dimension of A and B does not match")
    if a_mat.shape[0] < b_mat.shape[1]:
        raise ValueError("Dimension of A and B does not match")
    n = a_mat.shape[0]

    # calculate controllability matrix
    qc = b_mat
    for x in range(1, n):
        qc = np.concatenate((qc, np.linalg.matrix_power(a_mat, x) @ b_mat),
                            axis=1)

    # check controllability of the system
    if np.linalg.matrix_rank(qc) != n:
        raise ValueError("System is not controllable")

    return qc


def observability_matrix(a_mat, c_mat):
    """
    Calculate observability matrix and check observability of the system.

    .. math::
        \\boldsymbol{Q_{o}} = \\begin{pmatrix}
        \\boldsymbol{C}\\\\
        \\boldsymbol{C}\\boldsymbol{A}\\\\
        \\boldsymbol{C}\\boldsymbol{A}^{2}\\\\
        \\vdots \\\\
        \\boldsymbol{C}\\boldsymbol{A}^{n-1}\\\\
        \\end{pmatrix}

    Args:
        a_mat (:obj:`numpy.ndarray`): system matrix
        c_mat (:obj:`numpy.ndarray`): output matrix

    Return:
        :obj:`numpy.ndarray`: observability matrix :math:`\\boldsymbol{Q_{o}}`
    """
    a_mat = np.atleast_2d(a_mat)
    c_mat = np.atleast_2d(c_mat)

    # check dimension of matrix A and C
    if a_mat.shape[0] != a_mat.shape[1]:
        raise ValueError("A is not square")
    if a_mat.shape[0] != c_mat.shape[1]:
        raise ValueError("Dimension of A and C does not match")
    if a_mat.shape[0] < c_mat.shape[0]:
        raise ValueError("Dimension of A and C does not match")
    n = a_mat.shape[0]

    # calculate observability matrix
    qo = c_mat
    for x in range(1, n):
        qo = np.concatenate((qo, c_mat @ np.linalg.matrix_power(a_mat, x)),
                            axis=0)

    # check observability of the system
    if np.linalg.matrix_rank(qo) != n:
        raise ValueError("System is not observable")

    return qo


def place_siso(a_mat, b_mat, poles):
    """
    Place poles for single input single output (SISO) systems:

        - pol placement for state feedback: :math:`A` and :math:`b`
        - pol placement for observer: :math:`A^T` and :math:`c`

    Args:
        a_mat (:obj:`numpy.ndarray`): System matrix.:math:`A`
        b_mat (:obj:`numpy.ndarray`): Input vector :math:`b` or Output matrix
            :math:`c` .
        poles (list or :obj:`numpy.ndarray`): Desired poles.

    Return:
        :obj:`numpy.ndarray`: Feedback vector or :math:`k` or observer gain
        :math:`l^T` .
    """

    # check consistency
    if a_mat.shape[0] != a_mat.shape[1]:
        raise ValueError("A is not square")
    n = a_mat.shape[0]
    if n != b_mat.shape[0]:
        raise ValueError("Dimension of A and B does not match")
    m = b_mat.shape[1]
    if m != 1:
        raise ValueError("Dimension of B implies that is not a SISO system")
    if n != len(poles):
        raise ValueError("Dimension of A and the number of poles doesn't match")

    p = char_coefficients(poles)

    # calculate controllability matrix
    q = controllability_matrix(a_mat, b_mat)
    q_inv = np.linalg.inv(q)

    # last row in the inverse controllability matrix
    t1_t = np.atleast_2d(q_inv[-1])

    cm = np.linalg.matrix_power(a_mat, n)
    for i in range(n):
        cm = cm + p[i] * np.linalg.matrix_power(a_mat, i)

    k = np.dot(t1_t, cm)

    return k


def calc_prefilter(a_mat, b_mat, c_mat, k_mat=None):
    """
    Calculate the prefilter matrix

    .. math::
        \\boldsymbol{V} = - \\left[\\boldsymbol{C} \\left(\\boldsymbol{A} -
        \\boldsymbol{B}\\boldsymbol{K}\\right)^{-1}\\boldsymbol{B}\\right]^{-1}

    Args:
        a_mat (:obj:`numpy.ndarray`): system matrix
        b_mat (:obj:`numpy.ndarray`): input matrix
        c_mat (:obj:`numpy.ndarray`): output matrix
        k_mat (:obj:`numpy.ndarray`): control matrix

    Return:
        :obj:`numpy.ndarray`: Prefilter matrix
    """
    a_mat = np.atleast_2d(a_mat)
    b_mat = np.atleast_2d(b_mat)
    c_mat = np.atleast_2d(c_mat)
    k_mat = np.atleast_2d(k_mat)

    # check dimension of matrices A, B and C
    if a_mat.shape[0] != a_mat.shape[1]:
        raise ValueError("A is not square")
    if a_mat.shape[0] != b_mat.shape[0]:
        raise ValueError("Dimension of A and B does not match")
    if a_mat.shape[0] < b_mat.shape[1]:
        raise ValueError("Dimension of A and B does not match")
    if a_mat.shape[0] != c_mat.shape[1]:
        raise ValueError("Dimension of A and C does not match")
    if a_mat.shape[0] < c_mat.shape[0]:
        raise ValueError("Dimension of A and C does not match")

    if k_mat[0, 0] is not None:
        try:
            # prefilter: V = -[C(A-BK)^-1*B]^-1
            v = -mat_inv(c_mat @ (mat_inv(a_mat - b_mat @ k_mat) @ b_mat))
        except np.linalg.linalg.LinAlgError:
            raise ValueError("Cannot calculate V due to singularity.")
    else:
        v = np.array([[1]])

    return v



import sys
import os
import unittest
import subprocess
import re


class TestExamples(unittest.TestCase):
    """ Test Suite for the included examples"""

    def test_ballbeam(self):
        self._test_example("ballbeam", debug=False)

    def test_balltube(self):
        self._test_example("balltube", debug=False)

    def test_simple_pendulum(self):
        self._test_example("simple_pendulum", debug=False)

    def test_pendulum(self):
        self._test_example("pendulum", debug=False)

    def test_car(self):
        self._test_example("car", debug=False)

    def test_tanksystem(self):
        self._test_example("tanksystem", debug=True)

    def _test_example(self, name: str, debug=False):
        """
        Run the given example while listening for errors on stdout

        Args:
            name(str): Module name of the Example to run
            debug: If True, display all output from stdout. Pretty handy when
                things take longer.

        Raises:
            failureException: If the output contains log entries that mention
                "- ERROR -".
        """
        print("Testing Example: {}".format(name))
        pattern = re.compile(r"(- ERROR -)")
        script = os.sep.join([os.path.dirname(__file__), "example_runner.py"])
        cmd = [sys.executable, script, name]

        with subprocess.Popen(cmd,
                              stdout=subprocess.PIPE,
                              bufsize=1,
                              universal_newlines=True) as proc:
            for line in proc.stdout:
                if debug:
                    print(line, end="")
                if pattern.search(line):
                    proc.kill()
                    self.fail(line)

        if proc.returncode != 0:
            raise self.failureException("Process terminated with: {}"
                                        "".format(proc.returncode))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_registry
----------------------------------

Tests for `registry` module.
"""

import unittest

# nice top level import for these guys
from pymoskito import (register_simulation_module, register_processing_module,
                       register_visualizer, get_registered_simulation_modules,
                       get_registered_processing_modules)

# normal import since those should not be used so often from the outside
from pymoskito.registry import (get_registered_modules,
                                get_registered_visualizers)
from pymoskito.simulation_modules import *
from pymoskito.generic_simulation_modules import *
from pymoskito.processing_core import (ProcessingModule, PostProcessingModule,
                                       MetaProcessingModule)
from pymoskito.generic_processing_modules import *

from pymoskito.examples.ballbeam.model import BallBeamModel
from pymoskito.examples.ballbeam.controller import FController
from pymoskito.examples.ballbeam.postprocessing import BallBeamStepResponse
from pymoskito.examples.ballbeam.visualization import MplBallBeamVisualizer


class TestRegisterCalls(unittest.TestCase):
    def setUp(self):
        pass

    def test_generic_calls(self):
        """
        Check whether all generic simulation modules, postprocessing modules
        and visualizers are registered using the general getter call
        """

        # register a simulation module in wrong module category
        self.assertRaises(TypeError, register_simulation_module,
                          Controller, BallBeamModel)
        self.assertRaises(TypeError, register_processing_module,
                          PostProcessingModule, BallBeamModel)
        self.assertRaises(TypeError, register_visualizer, BallBeamModel)

        # register a postprocessor in wrong module category
        self.assertRaises(TypeError, register_simulation_module,
                          PostProcessingModule, BallBeamStepResponse)
        self.assertRaises(TypeError, register_processing_module,
                          MetaProcessingModule, BallBeamStepResponse)
        self.assertRaises(TypeError, register_visualizer, BallBeamStepResponse)

        # register a visualizer in wrong module category
        self.assertRaises(TypeError, register_simulation_module,
                          Model, MplBallBeamVisualizer)
        self.assertRaises(TypeError, register_processing_module,
                          PostProcessingModule, MplBallBeamVisualizer)

        # check registration
        # (the modules are registered if the import was successful)
        self.assertTrue((BallBeamModel, "BallBeamModel")
                        in get_registered_simulation_modules(Model))
        self.assertTrue((FController, "FController")
                        in get_registered_simulation_modules(Controller))
        self.assertTrue(
            (BallBeamStepResponse, "BallBeamStepResponse")
            in get_registered_processing_modules(PostProcessingModule))
        # self.assertTrue(
        #     (XYMetaProcessor, "XYMetaProcessor")
        #     in get_registered_processing_modules(MetaProcessingModule))
        self.assertTrue(
            (MplBallBeamVisualizer, "MplBallBeamVisualizer")
            in get_registered_visualizers())

        # test for automatic duplicate recognition
        self.assertRaises(ValueError, register_simulation_module,
                          Model, BallBeamModel)
        self.assertRaises(ValueError, register_simulation_module,
                          Controller, FController)
        self.assertRaises(ValueError, register_processing_module,
                          PostProcessingModule, BallBeamStepResponse)
        # self.assertRaises(ValueError, register_processing_module,
        #                   MetaProcessingModule, XYMetaProcessor)
        self.assertRaises(ValueError, register_visualizer,
                          MplBallBeamVisualizer)


class TestGetterCalls(unittest.TestCase):
    def setUp(self):
        # registration of generic modules is done in __init__.py
        pass

    def test_generic_calls(self):
        """
        check whether all generic simulation and postprocessing modules are
        registered using the general getter call
        """
        # ----------------------
        # simulation

        # solver
        self.assertTrue((ODEInt, "ODEInt")
                        in get_registered_modules(SimulationModule, Solver))

        # trajectory generators
        self.assertTrue((SmoothTransition, "SmoothTransition")
                        in get_registered_modules(SimulationModule, Trajectory))
        self.assertTrue((HarmonicTrajectory, "HarmonicTrajectory")
                        in get_registered_modules(SimulationModule, Trajectory))
        self.assertTrue((Setpoint, "Setpoint")
                        in get_registered_modules(SimulationModule, Trajectory))

        # controllers
        self.assertTrue((PIDController, "PIDController")
                        in get_registered_modules(SimulationModule, Controller))

        # mixers
        self.assertTrue((AdditiveMixer, "AdditiveMixer")
                        in get_registered_modules(SimulationModule, ModelMixer))
        self.assertTrue((AdditiveMixer, "AdditiveMixer")
                        in get_registered_modules(SimulationModule, ObserverMixer))

        # limiter
        self.assertTrue((ModelInputLimiter, "ModelInputLimiter")
                        in get_registered_modules(SimulationModule, Limiter))

        # sensors
        self.assertTrue((DeadTimeSensor, "DeadTimeSensor")
                        in get_registered_modules(SimulationModule, Sensor))

        # disturbance
        self.assertTrue((GaussianNoise, "GaussianNoise")
                        in get_registered_modules(SimulationModule, Disturbance))

        # ----------------------
        # processing

        # post-processors
        self.assertTrue((PlotAll, "PlotAll")
                        in get_registered_modules(ProcessingModule,
                                                  PostProcessingModule))
        # meta-processors
        # self.assertTrue((XYMetaProcessor, "XYMetaProcessor")
        #                 in get_registered_modules(ProcessingModule,
        #                                           MetaProcessingModule))

    def test_special_call(self):
        """
        special calls, same modules but different functions for access
        """
        # ----------------------
        # simulation

        self.assertTrue((PIDController, "PIDController")
                        in get_registered_simulation_modules(Controller))
        self.assertTrue((SmoothTransition, "SmoothTransition")
                        in get_registered_simulation_modules(Trajectory))
        self.assertTrue((HarmonicTrajectory, "HarmonicTrajectory")
                        in get_registered_simulation_modules(Trajectory))
        self.assertTrue((Setpoint, "Setpoint")
                        in get_registered_simulation_modules(Trajectory))

        # ----------------------
        # processing

        self.assertTrue(
            (PlotAll, "PlotAll")
            in get_registered_processing_modules(PostProcessingModule))
        # self.assertTrue(
        #     (XYMetaProcessor, "XYMetaProcessor")
        #     in get_registered_processing_modules(MetaProcessingModule))

    def test_string_call(self):
        """
        the calls should also work with names instead of class objects
        """
        self.assertTrue(
            (ODEInt, "ODEInt")
            in get_registered_modules("SimulationModule", "Solver"))
        self.assertTrue(
            (PlotAll, "PlotAll")
            in get_registered_processing_modules("PostProcessingModule"))

    def test_visualizer_call(self):
        """
        test interface for visualizer
        """
        self.assertTrue((MplBallBeamVisualizer, "MplBallBeamVisualizer")
                        in get_registered_visualizers())

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



import pymoskito.registry as pm
from pymoskito.processing_core import MetaProcessingModule
from pymoskito.generic_processing_modules import XYMetaProcessor


# TODO
class ErrorProcessor(XYMetaProcessor):

    def __init__(self):
        XYMetaProcessor.__init__(self,
                                 ["modules", "Controller"],
                                 ["modules", "Solver", "end time"],
                                 ["metrics", "L1NormITAE"])

